function [results]=imdcmelement(filename,parameters)
	mode(-1);

	MAX_PARAM=25;

	//File extension reading
	extension=part(filename,[length(filename)-2,length(filename)-1,length(filename)]);
	if extension<>'dcm' then
		disp('File '+string(parameters(i))+' is not a DICOM file!');
		results='err';
		return;
	end

	parameters_size=size(parameters);

	if parameters_size(2)<>1 then
		disp('Wrong parameters matrix!');
		results='err';
		return;
	end

	// To make interface easy, parameters are converted to an integer matrix
	// First line : 0 if this parameter is required
	// First line : 1 if this parameter is not required
	// Second line : order of parameter calling
	// Ex : if parameters=['inversion time';'slice thickness'] then
	// param_tab= ! 1 1 ...!
	//            ! 2 1 ...!
	param_tab=zeros(2,MAX_PARAM);
	for i=1:size(parameters,'*')
		convstr(parameters(i),'l');
		if parameters(i)=='slice thickness' then
			param_tab(:,1)=[1;i];
		elseif parameters(i)=='inversion time' then
			param_tab(:,2)=[1;i];
		elseif parameters(i)=='echo time' then
			param_tab(:,3)=[1;i];
		elseif parameters(i)=='repetition time' then
			param_tab(:,4)=[1;i];
		elseif parameters(i)=='number of averages' then
			param_tab(:,5)=[1;i];
		elseif parameters(i)=='imaging frequency' then
			param_tab(:,6)=[1;i];
		elseif parameters(i)=='matrix size' then
			param_tab(:,7)=[1;i];
		elseif parameters(i)=='field of view' then
			param_tab(:,8)=[1;i];
		elseif parameters(i)=='total receiver gain' then
			param_tab(:,9)=[1;i];
		elseif parameters(i)=='reconstruction scale factor' then
			param_tab(:,10)=[1;i];
		elseif parameters(i)=='transmitter calibration' then
			param_tab(:,11)=[1;i];
		elseif parameters(i)=='study date' then
			param_tab(:,12)=[1;i];
		elseif parameters(i)=='serial number' then
			param_tab(:,13)=[1;i];
		elseif parameters(i)=='image number' then
			param_tab(:,14)=[1;i];
		elseif parameters(i)=='real acquisition time' then
			param_tab(:,15)=[1;i];
		elseif parameters(i)=='real image time' then
			param_tab(:,16)=[1;i];
		elseif parameters(i)=='slice location' then
			param_tab(:,17)=[1;i];
		elseif parameters(i)=='field of view h' then
			param_tab(:,18)=[1;i];
		elseif parameters(i)=='field of view w' then
			param_tab(:,19)=[1;i];
		elseif parameters(i)=='image position vector s' then
			param_tab(:,20)=[1;i];
		elseif parameters(i)=='image position vector c' then
			param_tab(:,21)=[1;i];
		elseif parameters(i)=='image position vector t' then
			param_tab(:,22)=[1;i];
		elseif parameters(i)=='image normal cosinus s' then
			param_tab(:,23)=[1;i];
		elseif parameters(i)=='image normal cosinus c' then
			param_tab(:,24)=[1;i];
		elseif parameters(i)=='image normal cosinus t' then
			param_tab(:,25)=[1;i];
		elseif parameters(i)=='all' then
			param_tab=ones(2,MAX_PARAM);
			param_tab(2,:)=1:MAX_PARAM;
			parameters_size(1)=MAX_PARAM;;
		else
			disp('Wrong parameter : '+string(parameters(i))+' !');
			results='err';
			return;
		end
	end

	[tmp,err]=fileinfo(filename);
	if err==0 then
		[results]=dcmelement(filename,param_tab(:,:),parameters_size(1));
	else
		disp('File '+string(filename)+' doesn t exist!');
		results='err';
		return;
	end
endfunction
