function [region,nb_iter] = imgrowing(image,mask,points,threshold1,threshold2,size_seed,seeds_choice)
//scilab function for C function of growing region
//input:
//  image: original image
//  mask: binary image, where pixels should not be agregated to the region
//  points: coordinates of the seeds of the region
//  threshold1, threshold2
//  size_seed: integer, if seeds are single points, defines the size of square seed (size_seed*size_seed)
//  seeds_choice: 1 for single seeds, 2 if the whole

//output:
//  region: binary image, mask of the region of pixels agregated
//  nb_iter: number of iterations done during the process (<100)
//-----------------------------------------------------------------------------------------------------------------------
mode(-1);

//verify number of arguments
//--------------------------
[lhs,rhs]=argn(0);
if rhs<7 then
	disp("Wrong number of rhs arguments !");
	region='err';
	return;
end

//verify seeds_choice
//-------------------
if seeds_choice<>1 & seeds_choice<>2
	disp("wrong def of seeds_choice, choice must be 1 or 2");
	region='err';
	return;
end
if seeds_choice==2
	size_seeds=1
end

//verify sizes of image and mask
//------------------------------
image_size	= size(image);
mask_size	= size(mask);
if (mask_size(1)<>image_size(1)) | (mask_size(2)<>image_size(2)) then
	disp("Image and mask must have the same size!");
	region='err';
	return;
end

//verify size of points
//---------------------
nb_pts=size(points);
if nb_pts(:,1)<>2 then
	disp("points tab must have 2 lines (xi and yi) ");
	region='err';
	return;
end

//verify size_seed
//----------------
if((size_seed+1)/2<>ceil(size_seed/2))
	disp("size_seed must be odd");
	region='err';
	return;
end

//call to C function
//------------------
if (type(image)==1 & type(mask)==1 ) then
	[region,nb_iter]	= growing(image(:,:),mask(:,:),points(:,:),threshold1,threshold2,size_seed,seeds_choice);
else
	disp('Image should have only one plan!');
	region='err';
	return;
end

endfunction
