// Modified as of 05-02-05
// (see imshowgreylevel.sci)

function []=imshowrgb(image,cursor)
	mode(-1);

	set("figure_style","old");

	image_size=size(image);
	nbcolors=65536;
	width=image_size(1);
	height=image_size(2);

	// image256 has integer values between 0 and 255
	image256=im256(image);

	// Clear current graphic window
	xbasc();

	if (width<400) & (height<300) then
		xset('wdim',398, 298); // Scilab graphic window minimum size
		xsetech(wrect=[0,0,1,1],frect=[0,-1,width+1,height],arect=[(398-width)/(398*2),(398-width)/(398*2),(298-height)/(298*2),(298-height)/(298*2)]);
	else
		xset('wdim',width, height);
		xsetech(wrect=[0,0,1,1],frect=[0,-1,width+1,height],arect=[0,0,0,0]);
		if (width>900) | (height>600)
			xset('wpos',1,1)
		end
	end

	// Colormap creation
	// Try to allocate 256 colors
	ierr=execstr('xset(''colormap'',graycolormap(256))','errcatch');

	// In case of error, we allocate 253 colors
	// 256 colors platform
	// 253 grey levels + black + white + red (to draw ROIs)
	if ierr<>0
		disp('256 colors platform!');
		[image8 rmap gmap bmap] = createcmap128colors(image256(:,:,1), image256(:,:,2), image256(:,:,3));
	else
		[image8 rmap gmap bmap] = createcmap256colors(image256(:,:,1), image256(:,:,2), image256(:,:,3));
	end

	xset('wresize',0);
	// Pixmap mode => faster
	xset('pixmap',1);

	// Display
	if (nbcolors==256) then
		// Color loading
		cmap=[(rmap./127)',(gmap./127)',(bmap./127)'];
		xset('colormap',cmap);
		Matplot1(image8'+1,rect=[0,-1,width+1,height]);
	else
		// Color loading
		cmap=[rmap'/256,gmap'/256,bmap'/256];
		xset('colormap',cmap);
		Matplot1(image8'+1,rect=[0,-1,width+1,height]);
	end

	// Display pixmap
	xset('wshow');
	///if MSDOS then
	///	xbasr(xget('window')); // To avoid problems under Windows platform, but slower
	///end
	if cursor==1 then // To have a cursor on the image
		// Color level display
		while 1
			rep=xgetmouse(0);
			x=rep(1);
			y=rep(2);
			button=rep(3);
			if (x>=0)&(y<=height)&(y>0)&(x<width) then
				xcoord=floor(x)+1;
				ycoord=height-ceil(y)+1;
				r=image(xcoord,ycoord,1);
				g=image(xcoord,ycoord,2);
				b=image(xcoord,ycoord,3);
				position=sprintf('x=%d y=%d R=%f G=%f B=%f',xcoord,ycoord,r,g,b);
				xinfo(position);
			else
				// Can't exist if wresize=1
				xinfo('Out of image!');
			end;
			if button==5 then break,end // Right clic to exit
		end
	end

	// End of pixmap mode
	//xset('pixmap',0);

endfunction
