function [sdeviation_image] = imstandarddeviation(image,mykernel,mask)
	mode(-1);

	getf(MACROS+'verifyarguments.sci')

	// Read number of arguments
	[lhs,rhs] = argn(0);

	if rhs < 2
		error("Use : imstandarddeviation(image, mykernel [, mask])")
	end

	imageSize = size(image);
	imageDimension = size(imageSize);
	imageDimension = imageDimension(2);

	mat_type=type(image);

	// Verify mask good definition
	if rhs<3
		[mask, maskSize, maskDimension] = getmask(imageSize, imageDimension)
	else
		[maskSize, maskDimension] = verifymask(mask, imageSize, imageDimension)
	end

	// Verify kernel good definition
	msg = verifykernel(mykernel, imageSize)
	if msg <> ''
		error(msg)
	end

	if mat_type==1
		sdeviation_image = image_sdeviation(image, mykernel, mask);
	else
		error("The image should have only one plan!")
	end

endfunction

//................................................................................

function [msg] = verifykernel(mykern, imSize)

	mykernelSize = size(mykern);
	mykernelDim = size(mykernelSize);

	if ~(and(mykernelDim==[1 2])) then
		msg = "Bad kernel definition."
		// Verify if kernel has odd dimensions
	elseif mykernelSize(1)-int(mykernelSize(1)/2)*2==0
		msg = "Kernel must have odd dimensions."
	elseif mykernelSize(2)-int(mykernelSize(2)/2)*2==0
		msg = "Kernel must have odd dimensions."
	elseif (mykernelSize(1) > imSize(1)) | (mykernelSize(2) > imSize(2))
		msg = "Kernel is bigger than image!"
	else
		msg = ''
	end
endfunction

//................................................................................

function [mask, maskSize, maskDimension] = getmask(imageSize, imageDimension)

	if imageDimension == 2 | imageDimension == 3
		// 2D or 3D matrix
		mask = ones(image)
		maskSize = imageSize
		maskDimension = imageDimension
	elseif imageDimension == 4
		maskDimension = 3
		maskSize = imageSize(1:maskDimension)
		mask = ones(maskSize(1), maskSize(2), maskSize(3))
	else
		error('Bad dimension of the image')
	end

endfunction

