
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "hierarcluslib.h"

#define BigProto(i_fun, i_class, i_comp) bigProto.R[i_fun + i_class*inputSpaceDim + i_comp*inputSpaceDim*nbPoints]



// HierarClus object
HierarClus *hierarClus;


void hierarClusInit(int nNbComp, int nClassNumber, int nInputDimension) {
	// Creation of HierarClus object
	hierarClus = new HierarClus(nNbComp, nClassNumber, nInputDimension);
}

void hierarClusDeinit() {
	// Destruction of HierarClus object
	if (hierarClus)
		delete hierarClus;
}


int hierarClusTrain(HyperMat learningData, HyperMat proto) {
	// hierarchical clustering method training phase
	return hierarClus->train(learningData, proto);
}


//...................................................................................................


HierarClus::HierarClus(int nNbComp, int nClassNumber, int nInputDimension) :
				NeuralNet(nNbComp, nClassNumber, nInputDimension) {}


HierarClus::~HierarClus() {}



//################################################################################################################


int HierarClus::train(HyperMat learningData, HyperMat proto) {

	const double infini = 1e12;
	int nbPoints = learningData.dims[0];
	int classNb = nbPoints;

	HyperMat bigProto;
	double **distances;
	int *elementCount;
	double rAux, rMin;
	int i_class, i_class1, i_class2;
	int i_fun, i_comp;
	int numClass1, numClass2;
	int *iIndexes;


	// Memory allocation for bigProto and initialisation with learningData
	if (allocBigProto(&bigProto, learningData)) {
		return 1;
	}

	// Memory allocation for elementCount and initialisation : 1 point in each class
	if (elementCount = (int*) calloc(classNb, sizeof(int))) {
		for (i_class=0 ; i_class<classNb ; i_class++) {
			elementCount[i_class] = 1;
		}
	} else {
		return 1;
	}


	// Amalgamate the points to have classCount clusters

	if (GENERIC_NN_PROTO) {
		distances = (double **) calloc(classNb, sizeof (double *));
		if (distances) {
			distances[0] = (double *) calloc(classNb * classNb, sizeof (double));
			if (distances[0]) {
				for (i_class=1 ; i_class < classNb ; i_class++) {
					distances[i_class] = distances[0] + i_class * classNb;
				}

				while (classNb > classCount) {

					// Compute distances between all the classes prototypes
					for (i_class1=0 ; i_class1 < nbPoints ; i_class1++) {
						for (i_class2 = i_class1+1 ; i_class2 < nbPoints ; i_class2++) {
							if (elementCount[i_class1] && elementCount[i_class2]) {
								distances[i_class1][i_class2] = 0;

								for (i_fun = 0 ; i_fun < inputSpaceDim; i_fun++) {
									for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
										rAux = BigProto(i_fun, i_class1, i_comp)
											 - BigProto(i_fun, i_class2, i_comp);
										distances[i_class1][i_class2] += rAux * rAux;
									}
								}
								// Between-class variance
								distances[i_class1][i_class2] =
									(distances[i_class1][i_class2] * elementCount[i_class1] * elementCount[i_class2]) /
									(elementCount[i_class1] + elementCount[i_class2]);

							} else {
								distances[i_class1][i_class2] = infini;
							}
						}
					}

					// Choose the 2 classes that are the closest
					numClass1 = 0;
					numClass2 = 1;
					rMin = distances[0][1];

					for (i_class1=0 ; i_class1 < nbPoints ; i_class1++) {
						for (i_class2 = i_class1+1 ; i_class2 < nbPoints ; i_class2++) {
							if (rMin > distances[i_class1][i_class2]) {
								rMin = distances[i_class1][i_class2];
								numClass1 = i_class1;
								numClass2 = i_class2;
							}
						}
					}

					// Amalgamate the 2 classes
					for (i_fun=0 ; i_fun < inputSpaceDim ; i_fun++) {
						for (i_comp=0 ; i_comp < nbComp ; i_comp ++) {
							BigProto(i_fun, numClass2, i_comp) =
								( BigProto(i_fun, numClass2, i_comp) * elementCount[numClass2]
								+ BigProto(i_fun, numClass1, i_comp) * elementCount[numClass1] )
								/ (elementCount[numClass2] + elementCount[numClass1]);

						}
					}

					elementCount[numClass2] = elementCount[numClass2] + elementCount[numClass1];
					elementCount[numClass1] = 0;

					classNb--;

				}

				free (distances[0]);

				iIndexes = (int *) calloc(classCount, sizeof (int));
				if (iIndexes) {

					// Gathering data
					for (i_class1=0 , i_class=0 ; i_class1 < nbPoints ; i_class1++) {
						if (elementCount[i_class1]) {
							iIndexes[i_class] = i_class1;
							for (i_fun=0 ; i_fun < inputSpaceDim ; i_fun++) {
								for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
									Proto(i_fun, i_class, i_comp) = BigProto(i_fun, i_class1, i_comp);
								}
							}
							i_class++;
						}
					}

					free (iIndexes);
				}
			}
			free (distances);
		}
	}

	// Free memory
	freeBigProto(&bigProto);

	if (elementCount) {
		free(elementCount);
	}

	// Sort prototype classes in increasing order
	sortProto(proto);

	return 0;
}

//----------------------------------------------------------------------------------------

int HierarClus::allocBigProto(HyperMat *bigProto, HyperMat learningData) {

	int nbPoints = learningData.dims[0];
	const int nbDim = 3;
	int dimvar = 0;
	int i_fun, i_class, i_comp;


	bigProto->dimsize = nbDim;
	if (bigProto->dims = (int *) calloc(bigProto->dimsize, sizeof(int))) {
		bigProto->dims[dimvar++] = inputSpaceDim;
		bigProto->dims[dimvar++] = nbPoints;
		bigProto->dims[dimvar++] = nbComp;
	} else {
		return 1;
	}
	bigProto->size = inputSpaceDim * nbPoints * nbComp;
	if (bigProto->R = (double *) calloc(bigProto->size, sizeof(double))) {
		for (i_fun=0 ; i_fun<inputSpaceDim ; i_fun++) {
			for (i_class=0 ; i_class<nbPoints ; i_class++) {
    			for (i_comp=0 ; i_comp<nbComp ; i_comp++) {
					bigProto->R[i_fun + i_class*inputSpaceDim + i_comp*inputSpaceDim*nbPoints] =
						LearningData(i_class, i_fun, i_comp);
				}
			}
		}
	} else {
		return 1;
	}

	return 0;

}

//----------------------------------------------------------------------------------------

void HierarClus::freeBigProto(HyperMat *bigProto) {

	if (bigProto->dims) {
		free(bigProto->dims);
	}
	if (bigProto->R) {
		free(bigProto->R);
	}
}

//----------------------------------------------------------------------------------------




