#include "kmeanslib.h"

#ifndef HIERARCLUSLIB_H
#define HIERARCLUSLIB_H

#ifdef __cplusplus


class HierarClus : public NeuralNet {

	public :

		HierarClus(int nNbComp, int nClassNumber, int nInputDimension);

		~HierarClus();

		/* This function carries out the hierarchical clustering method training phase. From learning data
			(previously ramdomly generated pixels), we obtain a prototype of the classes.*/
		int train(HyperMat learningData, HyperMat proto);

	private :

		int allocBigProto(HyperMat *bigProto, HyperMat learningData);

		void freeBigProto(HyperMat *bigProto);

};

extern "C" {

#endif
	/** These functions can be accessed by C functions */

	/**@name hierarClusInit
	 * @description This fonction is used to initialize an object that allows to access others functions
	 * @param nNbComp : number of component for each pixel
	 * @param nClassNumber : class number, which is the number of pixels clusters that we want
	 * 		to make from data images
	 * @param nInputDimension : dimension of the input space, which is the number of computed
	 *		images considered (original image, average image, standard deviation image, ...)
	 *
	 */
	void hierarClusInit(int nNbComp, int nClassNumber, int nInputDimension);


	/**@name hierarClusDeinit
	 * @description This fonction is used to desinitialize the object that allows to access others functions
	 *
	 */
	void hierarClusDeinit();


	/**@name hierarClusTrain
	 * @description This function carries out the hierarchical clustering method training phase.
	 *		From learning data (previously ramdomly generated pixels), we obtain a prototype of the classes.
	 * @param learningData : ramdomly generated pixels, with their different values in the computed images.
	 *		This data is obtained with 'imlearningdata' Scilab function.
	 * @param proto : result prototype of the classes.
	 */
	int hierarClusTrain(HyperMat learningData, HyperMat proto);


#ifdef __cplusplus

}

#endif

#endif

