
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "kmeanslib.h"



// Kmeans object
Kmeans *kmeans;


void kmeansInit(int nNbComp, int nClassNumber, int nInputDimension) {
	// Creation of Kmeans object
	kmeans = new Kmeans(nNbComp, nClassNumber, nInputDimension);
}

void kmeansDeinit() {
	// Destruction of Kmeans object
	if (kmeans)
		delete kmeans;
}


int kmeansTrain(HyperMat learningData, HyperMat proto, double threshold, int nbIter) {
	// K-means method training phase 
	return kmeans->train(learningData, proto, threshold, nbIter);
}

int kmeansClassify(HyperMat proto, HyperMat mask, double *weights, HyperMat images[NBIMAGES_MAX],
						HyperMat classes_image) {

	// K-means method clustering phase
	return kmeans->classify(proto, mask, weights, images, classes_image);
}


//...................................................................................................


Kmeans::Kmeans(int nNbComp, int nClassNumber, int nInputDimension) :
				NeuralNet(nNbComp, nClassNumber, nInputDimension) {

}


Kmeans::~Kmeans() {

}


//################################################################################################################


int Kmeans::train(HyperMat learningData, HyperMat proto, double stopThreshold, int nbIter) {

	int nbPoints= learningData.dims[0];
	int nbComp = proto.dims[2];
	int *classBelonging;
	int *elementCount;
	double *distances;
	double rAux, rError, rMin;
	int i_fun, i_point, i_comp, i_class, iIndex, iEpoch;
	int classB;
	bool bChanged = true;

	if (!nbPoints) {
		return 1;
	}

	if (!(elementCount = (int*) calloc(classCount, sizeof(int)))) {
		return 1;
	}

	if (flags & GENERIC_NN_PROTO) {
		classBelonging = (int *) calloc (nbPoints, sizeof (int));
		if (classBelonging) {
			distances = (double *) calloc (classCount, sizeof (double));
			if (distances) {


				//Prototype initialisation
				initProto(learningData, proto);


				iEpoch = 0;
				do {

					for (i_point = 0 ; i_point < nbPoints ; i_point++) {
						// distances computation
						for (i_class = 0; i_class < classCount; i_class++) {
							distances[i_class] = 0;
							for (i_fun = 0 ; i_fun < inputSpaceDim; i_fun++) {
								for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
									rAux = LearningData(i_point, i_fun, i_comp) - Proto(i_fun, i_class, i_comp);
									distances[i_class] += rAux * rAux ;
								}
							}
						}
						// choose the classe with the minimal distance
      					rMin = distances[0];
						iIndex = 0;
						for (i_class = 1; i_class < classCount; i_class++) {
							if (rMin > distances[i_class]) {
								rMin = distances[i_class];
								iIndex = i_class;
							}
						}
						classBelonging[i_point] = iIndex;
					}

					// Update prototype
					for (i_class = 0 ; i_class < classCount ; i_class++) {
						elementCount[i_class] = 0;
						for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
							for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
								PrivateProto(i_fun, i_class, i_comp) = 0.0;
							}
						}
					}

					for (i_point = 0 ; i_point < nbPoints ; i_point++) {
						classB = classBelonging[i_point];
						elementCount[classB]++;

						for (i_fun = 0; i_fun < inputSpaceDim; i_fun++) {
							for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
								PrivateProto(i_fun, classB, i_comp) += LearningData(i_point, i_fun, i_comp);
							}
						}
					}

					for (i_class = 0 ; i_class < classCount ; i_class++) {
						for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
							for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
								PrivateProto(i_fun, i_class, i_comp) = (elementCount[i_class] > 0) ?
									PrivateProto(i_fun, i_class, i_comp) / elementCount[i_class] : 0.0;
							}
						}
					}

					// difference between prototypes of 2 consecutive computations
					rError = 0.0;
					for (i_class = 0 ; i_class < classCount ; i_class++) {
						for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
							for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
								rAux = Proto(i_fun, i_class, i_comp) - PrivateProto(i_fun, i_class, i_comp);
								Proto(i_fun, i_class, i_comp) = PrivateProto(i_fun, i_class, i_comp);
								rError += rAux * rAux;
							}
						}
					}
					rError = sqrt(rError);
					bChanged = (rError > stopThreshold);
					iEpoch++;

				} while (bChanged || (iEpoch < nbIter));

				free (distances);
			}
		free (classBelonging);
		}
	} // if (flags ...)

	if (elementCount) {
		free(elementCount);
	}

	// Sort prototype classes in increasing order
	sortProto(proto);

	return 0;

}



//...................................................................................................

int Kmeans::classify(HyperMat proto, HyperMat mask, double *weights, HyperMat images[NBIMAGES_MAX],
						HyperMat classes_image) {

	double **value;
	int i_point, i_fun, i_comp;

	// Memory allocation
	if (value = (double **) calloc(inputSpaceDim, sizeof(double*))) {
		if (value[0] = (double *) calloc(inputSpaceDim * nbComp, sizeof(double))) {
			for (i_fun=1 ; i_fun<inputSpaceDim ; i_fun++) {
				value[i_fun] = value[0] + i_fun * nbComp;
			}
		} else {
			return 1;
		}
	} else {
		return 1;
	}

	// Classify each pixel of images and form a classes image
	for (i_point=0 ; i_point<mask.size ; i_point++) {
		if (mask.R[i_point]) {
			for (i_fun=0 ; i_fun<inputSpaceDim ; i_fun++) {
				for (i_comp=0 ; i_comp < nbComp ; i_comp++) {
					value[i_fun][i_comp] = weights[i_fun] * images[i_fun].R[i_point + i_comp * mask.size];
				}
			}
			classes_image.R[i_point] = (double) classifyValue(proto, value);
		} else {
			classes_image.R[i_point] = 0.0;
		}
	}

	// Free memory
	if (value) {
		if (value[0]) {
			free(value[0]);
		}
		free(value);
	}

	return 0;
}


//...................................................................................................


int Kmeans::classifyValue(HyperMat proto, double **value) {

	int i_fun, i_class, i_comp ;
	double rMin, rAux, *distances;
	int iIndex = 0;

	if (flags & GENERIC_NN_PROTO) {
		distances = (double *) calloc (classCount, sizeof (double));
		if (distances) {
			// Distances computation
			for (i_class = 0; i_class < classCount; i_class++) {
				distances[i_class] = 0;
				for (i_fun = 0 ; i_fun < inputSpaceDim; i_fun++) {
					for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
						rAux = value[i_fun][i_comp] - Proto(i_fun, i_class, i_comp);
						distances[i_class] += rAux * rAux;
					}
				}
			}

			// Choose the classe with the minimal distance
			rMin = distances[0];
			iIndex = 0;
			for (i_class = 1; i_class < classCount; i_class++) {
				if (rMin > distances[i_class]) {
					rMin = distances[i_class];
					iIndex = i_class;
				}
			}
			free(distances);
		}
	}

	return iIndex;
}


