#include "neuralnet.h"

#ifndef KOHONENLIB_H
#define KOHONENLIB_H

#ifdef __cplusplus



class KohonenSOM : public NeuralNet {

	public :

		KohonenSOM(int nNbComp, int nClassNumber, int nInputDimension);

		~KohonenSOM();

		/* This function carries out the kohonen SOM method training phase. From learning data
			(previously ramdomly generated pixels), we obtain a prototype of the classes.*/
		int train(HyperMat learningData, HyperMat proto, int nbIter);

};

extern "C" {

#endif
	/** These functions can be accessed by C functions */

	/**@name kohonenInit
	 * @description This fonction is used to initialize an object that allows to access others functions
	 * @param nNbComp : number of component for each pixel
	 * @param nClassNumber : class number, which is the number of pixels clusters that we want
	 * 		to make from data images
	 * @param nInputDimension : dimension of the input space, which is the number of computed
	 *		images considered (original image, average image, standard deviation image, ...)
	 *
	 */
	void kohonenInit(int nNbComp, int nClassNumber, int nInputDimension);


	/**@name kohonenDeinit
	 * @description This fonction is used to desinitialize the object that allows to access others functions
	 *
	 */
	void kohonenDeinit();


	/**@name kohonenTrain
	 * @description This function carries out the kohonen SOM method training phase.
	 *		From learning data (previously ramdomly generated pixels), we obtain a prototype of the classes.
	 * @param learningData : ramdomly generated pixels, with their different values in the computed images.
	 *		This data is obtained with 'imlearningdata' Scilab function.
	 * @param proto : result prototype of the classes.
	 * @param nbIter : minimal number of iterations during training computation. Default value used is 100.
	 */
	int kohonenTrain(HyperMat learningData, HyperMat proto, int nbIter);


#ifdef __cplusplus

}

#endif

#endif

