
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "neuralnet.h"


//implementation of the generic class for the core of any neural network


NeuralNet::NeuralNet(int nNbComp, int nClassNumber, int nInputDim) {

	nbComp = nNbComp;
	classCount = nClassNumber;
	inputSpaceDim = nInputDim;
	flags = 0;

 	allocPrivateProto();
}

//----------------------------------------------------------------------------------------

NeuralNet::~NeuralNet() {

	freePrivateProto();

}


//----------------------------------------------------------------------------------------

void NeuralNet::allocPrivateProto() {

	if (flags & GENERIC_NN_PROTO) {
		freePrivateProto();
	}

	const int nbDim = 3;
	int dimvar = 0;
	privateProto.dimsize = nbDim;
	if (privateProto.dims = (int *) calloc(privateProto.dimsize, sizeof(int))) {
		privateProto.dims[dimvar++] = inputSpaceDim;
		privateProto.dims[dimvar++] = classCount;
		privateProto.dims[dimvar++] = nbComp;
	} else {
		return;
	}
	privateProto.size = inputSpaceDim * classCount * nbComp;
	if (privateProto.R = (double *) calloc(privateProto.size, sizeof(double))) {
		flags |= GENERIC_NN_PROTO;
	}
}

//----------------------------------------------------------------------------------------

void NeuralNet::freePrivateProto() {

	if (privateProto.dims) {
		free(privateProto.dims);
	}
	if (privateProto.R) {
		free(privateProto.R);
	}
}

//----------------------------------------------------------------------------------------

void NeuralNet::sortProto(HyperMat proto, int component) {

	int i_class1, i_class2, i_comp, i_fun;
	double *auxBuf, temp1, temp2;

	if (!(auxBuf = (double *)calloc(nbComp, sizeof(double)))) {
		return;
	}

	//sorting the prototypes
	component = component % inputSpaceDim;

	for (i_class1 = 1; i_class1 < classCount; i_class1++) {
		for (i_class2 = 0; i_class2 < i_class1; i_class2++) {
			temp2 = 0.0;
			temp1 = 0.0;
			for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
				temp2 += Proto(component, i_class2, i_comp);
				temp1 += Proto(component, i_class1, i_comp);
			}
			if (temp2 > temp1) {
				for (i_fun = 0; i_fun < inputSpaceDim; i_fun++) {
					for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
						auxBuf[i_comp] = Proto(i_fun, i_class2, i_comp);
						Proto(i_fun, i_class2, i_comp) = Proto(i_fun, i_class1, i_comp);
						Proto(i_fun, i_class1, i_comp) = auxBuf[i_comp];
					}
				}
			}
		}
	}

	if (auxBuf) {
		free(auxBuf);
	}

}


//----------------------------------------------------------------------------------------

void NeuralNet::initProto(HyperMat learningData, HyperMat proto) {

	int nbPoints= learningData.dims[0];
	int nbComp = proto.dims[2];
	int *classBelonging;
	int *elementCount;
	double *distances;
	double rAux, rError, rMin;
	int i_fun, i_point, i_comp, i_class, iIndex, iEpoch;
	bool bChanged = true;
	double stopThreshold = 1e-3;
	int nbIter = 10;

	HyperMat privProto;
	int i_initproto = 0;
	int tryNb = 0;
	double distIntra;
	double distInter;
	const double infini = 1e12;
	double score = infini;
	int i_class1, i_class2;

	if (!nbPoints) {
		return;
	}

	if (allocPrivProto(&privProto, learningData)) {
		return;
	}

	if (!(elementCount = (int*) calloc(classCount, sizeof(int)))) {
		return;
	}

	if (flags & GENERIC_NN_PROTO) {
		if (!(classBelonging = (int *) calloc (nbPoints, sizeof (int)))) {
			return;
		}
		if (!(distances = (double *) calloc (classCount, sizeof (double)))) {
			return;
		}

		//Prototype initialisation
		i_initproto = initProtoKmeans(learningData, privProto, i_initproto);

		while ((i_initproto > 0) && (tryNb < 10)) {

			iEpoch = 0;
			do {

				distIntra = 0.0;

				for (i_point = 0 ; i_point < nbPoints ; i_point++) {
					// distances computation
					for (i_class = 0; i_class < classCount; i_class++) {
						distances[i_class] = 0;
						for (i_fun = 0 ; i_fun < inputSpaceDim; i_fun++) {
							for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
								rAux = LearningData(i_point, i_fun, i_comp) - PrivProto(i_fun, i_class, i_comp);
								distances[i_class] += rAux * rAux ;
							}
						}
					}
					// choose the classe with the minimal distance
					rMin = distances[0];
					iIndex = 0;
					for (i_class = 1; i_class < classCount; i_class++) {
						if (rMin > distances[i_class]) {
							rMin = distances[i_class];
							iIndex = i_class;
						}
					}
					classBelonging[i_point] = iIndex;

					distIntra += rMin;
				}

				// Update prototype
				for (i_class = 0 ; i_class < classCount ; i_class++) {
					elementCount[i_class] = 0;
					for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
						for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
							PrivateProto(i_fun, i_class, i_comp) = 0.0;
						}
					}
				}

				for (i_point = 0 ; i_point < nbPoints ; i_point++) {
					iIndex = classBelonging[i_point];
					elementCount[iIndex]++;

					for (i_fun = 0; i_fun < inputSpaceDim; i_fun++) {
						for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
							PrivateProto(i_fun, iIndex, i_comp) += LearningData(i_point, i_fun, i_comp);
						}
					}
				}

				for (i_class = 0 ; i_class < classCount ; i_class++) {
					for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
						for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
							PrivateProto(i_fun, i_class, i_comp) = (elementCount[i_class] > 0) ?
								PrivateProto(i_fun, i_class, i_comp) / elementCount[i_class] : 0.0;
						}
					}
				}

				// difference between prototypes of 2 consecutive computations
				rError = 0.0;
				for (i_class = 0 ; i_class < classCount ; i_class++) {
					for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
						for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
							rAux = PrivProto(i_fun, i_class, i_comp) - PrivateProto(i_fun, i_class, i_comp);
							PrivProto(i_fun, i_class, i_comp) = PrivateProto(i_fun, i_class, i_comp);
							rError += rAux * rAux;
						}
					}
				}
				rError = sqrt(rError);
				bChanged = (rError > stopThreshold);
				iEpoch++;

			} while (bChanged && (iEpoch < nbIter));


			// Score computation
			distInter = infini;
			for (i_class1=0 ; i_class1 < classCount ; i_class1++) {
				for (i_class2 = i_class1+1 ; i_class2 < classCount ; i_class2++) {
					rError = 0.0;
					for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
						for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
							rAux = PrivProto(i_fun, i_class1, i_comp) - PrivProto(i_fun, i_class2, i_comp);
							rError += (rAux * rAux);
						}
					}
					if (rError < distInter) {
						distInter = rError;
					}
				}
			}
			if (distInter > 0) {
				rAux = distIntra / distInter;
			} else {
				rAux = infini-1;
			}
			if (rAux < score) {
				score = rAux;
				// Save prototype
				for (i_class = 0 ; i_class < classCount ; i_class++) {
					for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
						for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
							Proto(i_fun, i_class, i_comp) = PrivProto(i_fun, i_class, i_comp);
						}
					}
				}
			}

			tryNb++;

			i_initproto = initProtoKmeans(learningData, privProto, i_initproto);
		}

		if (distances) {
			free(distances);
		}
		if (classBelonging) {
			free(classBelonging);
		}

	} // if (flags ...)

	// Free memory
	freePrivProto(&privProto);

	if (elementCount) {
		free(elementCount);
	}

}


//----------------------------------------------------------------------------------------

int NeuralNet::initProtoKmeans(HyperMat learningData, HyperMat proto, int index) {

	int i_fun, i_point, i_comp, i_class;
	int nbPoints = learningData.dims[0];
	int j;

	double **rMin, **rMax;
	int seedsNb = 0;
	double dist, **thresholdDist;
	bool tooClose = false, close = true;

	if (index >= nbPoints) {
		return -1;
	}

	////
	if (rMin = (double **)calloc(inputSpaceDim, sizeof(double *))) {
		if (rMin[0] = (double *)calloc(inputSpaceDim * nbComp, sizeof(double))) {
			for(j=1 ; j<inputSpaceDim ; j++) {
				rMin[j] = rMin[0] + j * nbComp;
			}
		} else {
			return -1;
		}
	} else {
		return -1;
	}
	////
	if (rMax = (double **)calloc(inputSpaceDim, sizeof(double *))) {
		if (rMax[0] = (double *)calloc(inputSpaceDim * nbComp, sizeof(double))) {
			for(j=1 ; j<inputSpaceDim ; j++) {
				rMax[j] = rMax[0] + j * nbComp;
			}
		} else {
			return -1;
		}
	} else {
		return -1;
	}
	////
	if (thresholdDist = (double **)calloc(inputSpaceDim, sizeof(double *))) {
		if (thresholdDist[0] = (double *)calloc(inputSpaceDim * nbComp, sizeof(double))) {
			for(j=1 ; j<inputSpaceDim ; j++) {
				thresholdDist[j] = thresholdDist[0] + j * nbComp;
			}
		} else {
			return -1;
		}
	} else {
		return -1;
	}
	////



	for (j=0 ; j<proto.size ; j++) {
		proto.R[j] = 0;
	}

	if (nbPoints > classCount) {

		// Minima and maxima research
		for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
			for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
				rMin[i_fun][i_comp] = LearningData(0, i_fun, i_comp);
				rMax[i_fun][i_comp] = rMin[i_fun][i_comp];

				for (i_point = 1; i_point < nbPoints; i_point++) {
					if (rMin[i_fun][i_comp] > LearningData(i_point, i_fun, i_comp) ) {
						rMin[i_fun][i_comp] = LearningData(i_point, i_fun, i_comp);
					}
					if (rMax[i_fun][i_comp] < LearningData(i_point, i_fun, i_comp) ) {
						rMax[i_fun][i_comp] = LearningData(i_point, i_fun, i_comp);
					}
				}
				thresholdDist[i_fun][i_comp] =
					(rMax[i_fun][i_comp] - rMin[i_fun][i_comp]) / (double)(4*classCount);
				thresholdDist[i_fun][i_comp] =
					thresholdDist[i_fun][i_comp] * thresholdDist[i_fun][i_comp];

			}
		}

		// First seed
		i_point = index;
		for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
			for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
				Proto(i_fun, seedsNb, i_comp) = LearningData(i_point, i_fun, i_comp);
			}
		}
		seedsNb++;
		i_point++;

		while((seedsNb < classCount) && (i_point < nbPoints)) {

			for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
				for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
					Proto(i_fun, seedsNb, i_comp) = LearningData(i_point, i_fun, i_comp);
				}
			}
			// Seeds must not be too close
			i_class = 0 ;
			tooClose = false;
			while (!tooClose && (i_class < seedsNb)) {
				close = true;
				i_fun = 0 ;
				while (close && (i_fun < inputSpaceDim)) {
					i_comp = 0 ;
					while (close && (i_comp < nbComp)) {
						dist = Proto(i_fun, i_class, i_comp) - Proto(i_fun, seedsNb, i_comp);
						dist = dist * dist;

						if (dist <= thresholdDist[i_fun][i_comp]) {
							close = close && true;
						} else {
							close = false;
						}
						i_comp++;
					}
					i_fun++;
				}
				tooClose = close;
				i_class++;
			}
			if (!tooClose) {
				seedsNb++;
			}
			i_point++;
		}

		if (seedsNb < classCount) {
			return -1;
		} else {
			return i_point;
		}

	} else {
		for (i_point = 0 ; i_point < nbPoints ; i_point++) {
			for (i_fun = 0 ; i_fun < inputSpaceDim ; i_fun++) {
				for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
					Proto(i_fun, i_point, i_comp) = LearningData(i_point, i_fun, i_comp);
				}
			}
		}
		return nbPoints;
	}
}

//----------------------------------------------------------------------------------------

int NeuralNet::allocPrivProto(HyperMat *privProto, HyperMat learningData) {

	int nbPoints = learningData.dims[0];
	const int nbDim = 3;
	int dimvar = 0;

	privProto->dimsize = nbDim;
	if (privProto->dims = (int *) calloc(privProto->dimsize, sizeof(int))) {
		privProto->dims[dimvar++] = inputSpaceDim;
		privProto->dims[dimvar++] = classCount;
		privProto->dims[dimvar++] = nbComp;
	} else {
		return 1;
	}
	privProto->size = inputSpaceDim * classCount * nbComp;
	if (!(privProto->R = (double *) calloc(privProto->size, sizeof(double)))) {
		return 1;
	}

	return 0;

}

//----------------------------------------------------------------------------------------

void NeuralNet::freePrivProto(HyperMat *privProto) {

	if (privProto->dims) {
		free(privProto->dims);
	}
	if (privProto->R) {
		free(privProto->R);
	}
}

