/******************************************************************************/
/* Functions used to create a colormap for a color image                      */
/* And to disply it with true colors                                          */
/* Using the algorithm of ppm2sci() function of IMAGE contribution            */
/* Warning : this function crashes Scilab if too many colors in image         */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"

#include "int_createcmap.h"

extern int compare(long *long1, long *long2);

/******************************************************************************/
/* Interface Scilab <-> createcmap                                            */
/******************************************************************************/
/* Creation : 25/02/2002 (V. COUVERT)                                         */
/* Last modification : 29/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_createcmap(char *fname)
{
	int l_redimage, m_redimage, n_redimage,
	    l_greenimage, m_greenimage, n_greenimage,
	    l_blueimage, m_blueimage, n_blueimage,
	    *l_image8, m_image8, n_image8,
	    *l_rmap, m_rmap, n_rmap,
	    *l_gmap, m_gmap, n_gmap,
	    *l_bmap, m_bmap, n_bmap;
	
    int ierr=0;

	int minlhs=4, maxlhs=4, minrhs=5, maxrhs=5;
	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */
	
	GetRhsVar(1, "i", &m_redimage, &n_redimage, &l_redimage);
	GetRhsVar(2, "i", &m_greenimage, &n_greenimage, &l_greenimage);
	GetRhsVar(3, "i", &m_blueimage, &n_blueimage, &l_blueimage);

	createcmap(istk(l_redimage), istk(l_greenimage), istk(l_blueimage), 
		m_redimage, n_redimage, &l_image8, &l_rmap, &l_gmap, &l_bmap, &n_rmap, &ierr);

	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	m_rmap=m_gmap=m_bmap=1;
	n_gmap=n_bmap=n_rmap;
	
	CreateVarFromPtr(4, "i", &m_redimage, &n_redimage, &l_image8);
	CreateVarFromPtr(5, "i", &m_rmap, &n_rmap, &l_rmap);
	CreateVarFromPtr(6, "i", &m_gmap, &n_gmap, &l_gmap);
	CreateVarFromPtr(7, "i", &m_bmap, &n_bmap, &l_bmap);

	LhsVar(1)=4;
	LhsVar(2)=5;
	LhsVar(3)=6;
	LhsVar(4)=7;

	if (l_image8) {
		free(l_image8);
	}
	if (l_rmap) {
		free(l_rmap);
	}
	if (l_gmap) {
		free(l_gmap);
	}
	if (l_bmap) {
		free(l_bmap);
	}

	
	return(0);
}

/******************************************************************************/
/* Creation function                                                          */
/******************************************************************************/
int createcmap(int *redimage, int *greenimage, int *blueimage, int w, int h, 
			   int **p8, int **rmap, int **gmap, int **bmap, int *cmap_size, int *ierr)
{
	int *rgb, j=0, x, y;
	long nc=0, *p24, *p24sort, redbuf, greenbuf, bluebuf, index, index_tab[65536];

	/* Memory allocation */
    if (!(*p8= (int*)calloc(1, sizeof(int)*w*h)))
    {
		*ierr=1;
		return(-1);
    }
	if (!(p24= (long*)calloc(1,sizeof(long)*w*h)))
    {
		*ierr=1;
		return(-1);
    }
	if (!(p24sort= (long*)calloc(1,sizeof(long)*w*h)))
    {
		*ierr=1;
		return(-1);
    }
	/* Values of pixels to display */
	for(y=0 ; y<h ; y++)
	{
		for(x=0 ; x<w ; x++)
		{
			redbuf=*(redimage+x+y*w);
			greenbuf=*(greenimage+x+y*w) << 8;
			bluebuf=*(blueimage+x+y*w) << 16;
			*(p24+x+y*w)=redbuf | greenbuf | bluebuf;
			*(p24sort+x+y*w)=redbuf | greenbuf | bluebuf;
		}
	}

	/* Sort the values in decreasing order*/
	qsort(p24sort,w*h,sizeof(long),compare);

	/* Searching for different values in p24sort */
	nc=0;
	for(index=0 ; index <w*h-1 ; index++)
	{
		if(*(p24sort+index)!=*(p24sort+index+1))
		{
			index_tab[nc]=index;
			nc++;
			/* If too many colors */
			if(nc>65535)
			{
				*ierr=1;
				return(-1);
			}
		}
	}

	/* Memory allocation */
	if (!(rgb = (int*)calloc(1, sizeof(int)*nc)))
    {
		*ierr=1;
		return(-1);
    }

	/* Filling rgb table */
	for(j=0 ; j<nc ; j++)
	{
		if(j==0)
		{
			rgb[0]=*(p24sort+index_tab[0]);
			rgb[1]=*(p24sort+index_tab[0]+1);
		}
		else
		{
			rgb[j+1]=*(p24sort+index_tab[j]+1);
		}
	}
	nc++;

	/* Memory allocation */
	if (!(*rmap = (int*)calloc(1, sizeof(int)*nc)))
    {
		*ierr=1;
		return(-1);
    }
    if (!(*gmap = (int*)calloc(1, sizeof(int)*nc)))
    {
		*ierr=1;
		return(-1);
    }
    if (!(*bmap = (int*)calloc(1, sizeof(int)*nc)))
    {
		*ierr=1;
		return(-1);
    }
	
	for(j=0 ; j<nc ; j++)
	{
		(*rmap)[j]=(rgb[j]) & 0xFF;
		(*gmap)[j]=((rgb[j]) & 0xFF00) >> 8;
		(*bmap)[j]=((rgb[j]) & 0xFF0000) >> 16;
	}

	/* Change values to display */
	for(j=0 ; j<nc ; j++)
	{
		for(y=0 ; y<h ; y++)
		{
			for(x=0 ; x<w ; x++)
			{
				if(*(p24+x+y*w)==*(rgb+j))
					(*p8)[x+y*w]=j;
			}
		}
	}

	*cmap_size=nc;
	free(p24sort);
	free(p24);
	free(rgb);

	*ierr=0;
	return(0);
}


