/******************************************************************************/
/* Functions used to create an image mask from Scilab                         */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "stack-c.h"
#include "int_createmask.h"

/******************************************************************************/
/* Interface Scilab <-> createmask                                            */
/******************************************************************************/
/* Creation : 16/08/2001 (V. COUVERT)                                         */
/* Last modification : 29/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_createmask(char *fname)
{
	int *l_mask, m_mask, n_mask,
	    l_width, m_width, n_width,
	    l_height, m_height, n_height,
	    l_xsel, m_xsel, n_xsel,
	    l_ysel, m_ysel, n_ysel,
	    l_nb_pts, m_nb_pts, n_nb_pts,
	    l_type, m_type, n_type;

	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=6, maxrhs=6;
	
	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */
	
	GetRhsVar(1, "c", &m_type, &n_type, &l_type);
	GetRhsVar(2, "i", &m_width, &n_width, &l_width);
	GetRhsVar(3, "i", &m_height, &n_height, &l_height);
	GetRhsVar(4, "i", &m_xsel, &n_xsel, &l_xsel);
	GetRhsVar(5, "i", &m_ysel, &n_ysel, &l_ysel);
	GetRhsVar(6, "i", &m_nb_pts, &n_nb_pts, &l_nb_pts);
	
	createmask(&l_mask, *cstk(l_type), *istk(l_width), *istk(l_height), istk(l_xsel), istk(l_ysel), *istk(l_nb_pts), &ierr);
	
	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	m_mask=*istk(l_width);
	n_mask=*istk(l_height);

	CreateVarFromPtr(7,"i", &m_mask, &n_mask, &l_mask);
	
	if (l_mask) {
		free(l_mask);
	}
	
	LhsVar(1)=7;

	return(0);
}

/******************************************************************************/
/* Mask creation function                                                     */
/******************************************************************************/
int createmask(int **mask, char type, int width, int height, int *xsel, int *ysel,
			   int nb_pt_def, int *ierr)
{
	int index=0,
	    radius, a, b, xcenter, ycenter, 
	    x, y, i, j=0, k, l=0, 
	    deltax=0, deltay=0, xpt1=0,ypt1=0, xpt2=0,ypt2=0;
	
	/* Memory allocation */
	if (!(*mask = (int*)calloc(1, sizeof(int)*width*height)))
	{
		*ierr=1;
		return(-1);
	}
	
	switch(type)
	{
		case 'r' :
			for (y = 0 ; y < height ; y++)
		{
			for (x = 0 ; x < width ; x++)
			{
				index=x+y*width;
				(*mask)[index]=1;
				if((x<=xsel[0]) || (x>=xsel[1]))
				{
					(*mask)[index]=0;
				}
				if((y<=ysel[0]) || (y>=ysel[1]))
				{
					(*mask)[index]=0;
				}
				}
			}
	    		break;
	    					
    	case 'c' :
    		xcenter=(xsel[1]+xsel[0])/2;
			ycenter=(ysel[1]+ysel[0])/2;
			radius=(xsel[1]-xsel[0])/2;
    		for (y = 0 ; y < height ; y++)
			{
				for (x = 0 ; x < width ; x++)
				{
					index=x+y*width;
					if(((x-xcenter)*(x-xcenter)+(y-ycenter)*(y-ycenter))>=radius*radius)
					{
						(*mask)[index]=0;
					}
					else
					{
						(*mask)[index]=1;
					}
				}
			}
			break;
							
		case 'e' :
			xcenter=(xsel[1]+xsel[0])/2;
			ycenter=(ysel[1]+ysel[0])/2;
			a=(xsel[1]-xsel[0])/2;
			b=(ysel[1]-ysel[0])/2;
			for (y = 0 ; y < height ; y++)
			{
				for (x = 0 ; x < width ; x++)
     			{
					index=x+y*width;
					if( ( ((x-xcenter)*(x-xcenter))*(b*b) + ((y-ycenter)*(y-ycenter))*(a*a) ) >= a*a*b*b)
					{
						(*mask)[index]=0;
					}
					else
					{
						(*mask)[index]=1;
					}
				}
			}
			break;
		
		case 'p' :
			/* Mask RAZ */
			for(i=0;i<width*height;i++)
			{
				(*mask)[i]=0;
			}
			for(i=0;i<nb_pt_def;i++)
			{
				ysel[i]=height-ysel[i];
			}
			i=0;
			do
			{
				if(ysel[i]<=ysel[i+1])
				{
					l=xpt1=xsel[i];
					ypt1=ysel[i];
					xpt2=xsel[i+1];
					ypt2=ysel[i+1];
					deltax=xpt2-xpt1;
					deltay=ypt2-ypt1;	
				}
				if(ysel[i]>ysel[i+1])
				{
					xpt2=xsel[i];
					ypt2=ysel[i];
					l=xpt1=xsel[i+1];
					ypt1=ysel[i+1];
					deltax=xpt2-xpt1;
					deltay=ypt2-ypt1;	
				}
				for(j=ypt1, k=0; j<=(ypt2); j++,k++)
				{
					if(deltay!=0)
					{
						(*mask)[j*width+(xpt1)+k*deltax/deltay]=1;
						if(deltax>=0)
						{
							while(l<=((xpt1)+k*deltax/deltay))
							{
								(*mask)[j*width+l]=1;
								l++;
							}
						}
						if(deltax<0)
						{
							while(l>=((xpt1)+k*deltax/deltay))
							{
								(*mask)[j*width+l]=1;
								l--;
							}
						}
					}
					if(deltay==0)
					{
						if(deltax>=0)
						{
							while(l<=(xpt2))
							{
								(*mask)[j*width+l]=1;
								l++;
							}
						}
						if(deltax<0)
						{
							while(l>=(xpt2))
							{
								(*mask)
								[j*width+l]=1;
								l--;
							}
						}
					}
				}  
				i++;

			}
			while(i<(nb_pt_def-1));
			xvw_scanline_fill(mask, width, height, 0, 1);
			break;
	    					
		default :
			*ierr=1;
			return(-1);
			break;
	}
	*ierr=0;
	return(0);
}

/******************************************************************************/
/* Flood filling function                                                     */
/******************************************************************************/
int xvw_flood_fill(int **mask, int  w, int  h, int  x, int  y, int  set, int  val)
{
        int j, k, indx = y*w;

        j = x-1; k = x+1;
        (*mask)[indx+x] = val;

        while (j >= 0 && (*mask)[indx+j] == set) (*mask)[indx + j--] = val;
        while (k < w  && (*mask)[indx+k] == set) (*mask)[indx + k++] = val;

        for (x = j+1, indx = j+1 + y*w; x < k; x++, indx++)
        {
           if (y > 0 && (*mask)[indx-w] == set)
              xvw_flood_fill(mask, w, h, x, y-1, set, val);
           if (y < h-1 && (*mask)[indx+w] == set)
              xvw_flood_fill(mask, w, h, x, y+1, set, val);
        }
}

/******************************************************************************/
/* Roi scanning and filling function                                          */
/******************************************************************************/
int xvw_scanline_fill(int **mask, int w, int h, int set, int val)
{
	int *tmp;
	int  i, j, put;

	for (i = 0, j = w*(h-1); i < w; i++, j++)
	{
	   if ((*mask)[i] == set) 	
		xvw_flood_fill(mask, w, h, i, 0, set, -1);
	   if ((*mask)[j] == set) 	
		xvw_flood_fill(mask, w, h, i, (int) h-1, set, -1);
	}
	for (i = 0, j = w-1; i < h; i++)
	{
	   if ((*mask)[i*w] == set) 
		xvw_flood_fill(mask, w, h, 0, i, set, -1);
	   if ((*mask)[i*w+j] == set) 
		xvw_flood_fill(mask, w, h, j, i, set, -1);
	}

	for (j = 0, tmp = *mask; j < h; j++)
	{
	   for (i = 0, put = 1; i < w; i++)
	   {
	      if (*tmp++ == set)
		 xvw_flood_fill(mask, w, h, i, j, set, !put ? -1 : val);
	   }
	}

	for (i = 0; i < w*h; i++)
	   (*mask)[i] = ((*mask)[i] == -1) ? set : (*mask)[i];
}





