/******************************************************************************/
/* Functions used to calculate an average image from Scilab                   */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"

#include "int_imaverage.h"

/******************************************************************************/
/* Interface Scilab <-> imaverage                                             */
/******************************************************************************/
/* Creation : 02/08/2001 (V. COUVERT)                                         */
/* Last modification : 23/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imaverage(char *fname)
{
	static int l_image, m_image, n_image,
	    l_kernel, m_kernel, n_kernel,
	    m_average_image, n_average_image,
	    l_mask, m_mask, n_mask;
	int ierr=0;
	double *l_average_image;

	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "d", &m_kernel, &n_kernel, &l_kernel);
	GetRhsVar(3, "d", &m_mask, &n_mask, &l_mask);

	average_image(stk(l_image), m_image, n_image, stk(l_kernel), m_kernel, n_kernel, &l_average_image, stk(l_mask), &ierr);

	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(4,"d", &m_image, &n_image, &l_average_image);

	if (l_average_image) {
		free(l_average_image);
	}

	LhsVar(1)=4;

	return(0);
}

/******************************************************************************/
/* Image average value computation function                                   */
/******************************************************************************/

int average_image(double *image, int width, int height, double *kernel, 
				  int kernel_width, int kernel_height, double ** average_image, 
				  double *mask, int *ierr)
{
	static int x, y, xscrut, yscrut, nb_val=0;
	double sum=0.0;

	/* Memory allocation */
	if (!(*average_image = (double*)calloc(width*height, sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}

	/* Filling average image */
	for(y=0 ; y<height ; y++)
	{
		for(x=0 ; x<width ; x++)
		{
			/* If the pixel is not masked */
			if(*(mask+x+y*width)!=0)
			{
				/* Find the minimum */
				for(yscrut=-kernel_height/2 ; yscrut<=kernel_height/2 ; yscrut++)
				{
					for(xscrut=-kernel_width/2 ; xscrut<=kernel_width/2 ; xscrut++)
					{
						/* If value has to be count : kernel corresponding value is != 0*/
						if(kernel[xscrut+kernel_width/2+(yscrut+kernel_height/2)*kernel_width]!=0)
						{
							/* If we point a pixel in the image (to take border in account) and the pixel is not masked*/
							if( (x+xscrut>=0) && (x+xscrut<width) && (y+yscrut>=0) && (y+yscrut<height) && ((*(mask+x+xscrut+(y+yscrut)*width))!=0) )
							{
								sum=sum+(*(image+x+y*width+xscrut+yscrut*width));
								nb_val++;
							}
						}
					}
				}
				if(nb_val!=0)
				{
					(*average_image)[x+y*width]=sum/nb_val;
				}
				else
				{
					(*average_image)[x+y*width]=0.0;
				}
				sum=0.0;
				nb_val=0;
			}
			else
			{
				(*average_image)[x+y*width]=*(image+x+y*width);
			}
		}
	}

	*ierr=0;
	return(0);
}
