/******************************************************************************/
/* Functions used to calculate an image convolution from Scilab               */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include "int_imconvol.h"

/******************************************************************************/
/* Interface Scilab <-> imconvol                                              */
/******************************************************************************/
/* Creation : 09/04/2002 (V. COUVERT)                                         */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imconvol(char *fname)
{
	int l_image, m_image, n_image,
	    l_kernel, m_kernel, n_kernel,
	    m_convol_image, n_convol_image,
	    l_mask, m_mask, n_mask;
	double *l_convol_image;

	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */
	
	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "d", &m_kernel, &n_kernel, &l_kernel);
	GetRhsVar(3, "d", &m_mask, &n_mask, &l_mask);

	convol_image(stk(l_image), m_image, n_image, stk(l_kernel), m_kernel, n_kernel, &l_convol_image, stk(l_mask), &ierr);

	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(4,"d", &m_image, &n_image, &l_convol_image);

	if (l_convol_image) {
		free(l_convol_image);
	}


	LhsVar(1)=4;

	return(0);
}

/******************************************************************************/
/* Image convolution computation function                                     */
/******************************************************************************/
/* Creation : 09/04/2001 (V. COUVERT)                                         */
/* Last modification : 23/04/2002 (V. COUVERT)                                */
/******************************************************************************/
int convol_image(double *image, int width, int height, double *kernel,
				 int kernel_width, int kernel_height, double **convol_image,
				 double *mask, int *ierr)
{
	static int x, y, xscrut, yscrut, nb_val=0;
	double sum=0.0;

	/* Memory allocation */
	if (!(*convol_image = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}

	/* Filling average image */
	for(y=0 ; y<height ; y++)
	{
		for(x=0 ; x<width ; x++)
		{
			/* If the pixel is not masked */
			if(*(mask+x+y*width)!=0)
			{
				/* Find the minimum */
				for(yscrut=-kernel_height/2 ; yscrut<=kernel_height/2 ; yscrut++)
				{
					for(xscrut=-kernel_width/2 ; xscrut<=kernel_width/2 ; xscrut++)
					{
						/* If value has to be count : kernel corresponding value is != 0*/
						if(kernel[xscrut+kernel_width/2+(yscrut+kernel_height/2)*kernel_width]!=0)
						{
							/* If we point a pixel in the image (to take border in account) and the pixel is not masked*/
							if( (x+xscrut>=0) && (x+xscrut<width) && (y+yscrut>=0) && (y+yscrut<height) && ((*(mask+x+xscrut+(y+yscrut)*width))!=0) )
							{
								sum=sum+( (*(image+x+y*width+xscrut+yscrut*width)) * (*(kernel+xscrut+kernel_width/2+(yscrut+kernel_height/2)*kernel_width)) );
								nb_val++;
							}
						}
					}
				}
				if(nb_val!=0)
				{
					(*convol_image)[x+y*width]=sum/nb_val;
				}
				else
				{
					(*convol_image)[x+y*width]=0;
				}
				sum=0;
				nb_val=0;
			}
			else
			{
				(*convol_image)[x+y*width]=*(image+x+y*width);
			}
		}
	}

	*ierr=0;
	return(0);
}
