/******************************************************************************/
/* Functions used to read a dicom file header from Scilab                     */
/******************************************************************************/

#include <malloc.h>
#include "stack-c.h"
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>

#include "int_imdcmheader.h"




/******************************************************************************/
/* Interface Scilab <-> imdcmheader                                           */
/******************************************************************************/
/* Creation : 18/06/2002 (V. COUVERT)                                         */
/* Last modification : 23/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imdcmheader(char *fname)
{
	int l_filename, m_filename, n_filename,
	    l_output_file, m_output_file, n_output_file;

	int ierr=0;

	int minlhs=0, maxlhs=1, minrhs=2, maxrhs=2;

	CheckRhs(minrhs, maxrhs); // Input arguments 
	CheckLhs(minlhs, maxlhs); // Output arguments 

	GetRhsVar(1, "c", &m_filename, &n_filename, &l_filename);

	GetRhsVar(2, "c", &m_output_file, &n_output_file, &l_output_file);

	dcm_header(cstk(l_filename), cstk(l_output_file), &ierr);

	// In case of error
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	return(0);
}

/******************************************************************************/
/* Header file reading function                                               */
/******************************************************************************/
/* Creation : 18/06/2001 (V. COUVERT)                                         */
/* Last modification : 23/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int dcm_header(char *filename, char *output_file, int *ierr)
{
	DCM_OBJECT *dcmObject = NULL;
	DCM_ELEMENT dcmElement;
	CONDITION cond;
	PRV_GROUP_ITEM *groupItem;
	PRIVATE_OBJECT **object;
	unsigned long nOptions = DCM_ORDERLITTLEENDIAN | DCM_PART10FILE;
	void *dummPointer;
	U32 rtnSize;
	void *ctx=NULL;
	int id=-1;
	char *buf[100];

	// Virginie
	int lg;

	// Openning the DICOM file
	cond = DCM_OpenFile (filename, nOptions, &dcmObject);

	if(cond!=DCM_NORMAL) /* Opening file failed : maybe it is not a PART10 file... */
	{
		nOptions &= ~(DCM_PART10FILE);
		cond = DCM_OpenFile (filename, nOptions, &dcmObject);
		if(cond!=DCM_NORMAL) /* Opening file has failed : try with BIGENDIANORDER */
		{
			nOptions |= DCM_PART10FILE;
			nOptions &= ~(DCM_ORDERMASK);
			nOptions |= DCM_ORDERBIGENDIAN;
			cond = DCM_OpenFile (filename, nOptions, &dcmObject);
			if(cond!=DCM_NORMAL) /* Opening file failed : maybe it is not a PART10 file... */
			{
				nOptions &= ~(DCM_PART10FILE);
				cond = DCM_OpenFile (filename, nOptions, &dcmObject);
				if(cond!=DCM_NORMAL)
				{
					sciprint("File %s doesn't exist or read access denied.\n", filename);
					*ierr=1;
					return(-1);
				}
			}
		}
	}
	else
	{
		// Try to create output file 
		id=open(output_file, O_WRONLY | O_CREAT | O_BINARY, S_IREAD | S_IWRITE);
		if(id==-1) // If creation fails 
		{
			sciprint("Can't create file %s.\n", output_file);

			// Virginie
			DCM_CloseObject(&dcmObject);			

			*ierr=1;
			return(1);
		}
		// Writing image file name in output file

		lg = sprintf(buf,"%sHeader from file : %s",LINE_STRING,filename);
		write(id,buf,lg);

		lg = sprintf(buf,"\nThis file has a PART10 !",LINE_STRING,filename);
		write(id,buf,lg);

	}

	if(id==-1) /* If the file is a part10 file with little endian order */
	{
		/* Try to create output file */
		id=open(output_file, O_WRONLY | O_CREAT | O_BINARY, S_IREAD | S_IWRITE);
		if(id==-1) /* If creation fails */
		{
			sciprint("Can't create file %s.\n", output_file);

			DCM_CloseObject(&dcmObject);			

			*ierr=1;
			return(1);
		}
		/* Writing image file name in output file */
		sprintf(buf,"%sHeader from file : %s",LINE_STRING,filename);
		write(id,buf,strlen(buf));
	}
	lg = sprintf(buf,"%s",LINE_STRING);
	write(id,buf,lg);

	object = (PRIVATE_OBJECT **) &dcmObject;

	// Writing data in file

	// groupItem pointe sur la tete de la liste des groupes
	groupItem = LST_Head(&(*object)->groupList);

	// Place le curseur de la liste des groupes en tete de liste des groupes ? 
	if (groupItem != NULL) {
		(void) LST_Position(&(*object)->groupList, groupItem);
	}

	// Boucle sur tous les groupes de la liste 
	while (groupItem != NULL)
	{
		sprintf(buf,"%sGROUP  %04x%s\n",LINE_STRING,groupItem->group,LINE_STRING);
		write(id,buf,strlen(buf));
		print_list_elem(id,groupItem->elementList);
		groupItem = LST_Next(&(*object)->groupList);
	}

	// Closing ouput file
	close(id);

	// Closing DICOM file
	DCM_CloseObject(&dcmObject);
	*ierr=0;
	return(0);
}

/******************************************************************************/
/* Elements of groups reading function                                        */
/******************************************************************************/
/* Creation : 18/06/2001 (V. COUVERT)                                         */
/* Last modification : 18/06/2002 (V. COUVERT)                                */
/******************************************************************************/
int print_list_elem(int id, LST_HEAD *elementList)
{
	PRV_ELEMENT_ITEM *elementItem;
	char scratch[128];
	int stringLength;
	char buffer[100];

	/* elementItem pointe sur la tete de la liste des elements */
	elementItem = LST_Head(&elementList);

	/* Place le curseur de la liste des elements en tete de liste des elements ? */
	if (elementItem != NULL)
		(void) LST_Position(&elementList,elementItem);

	/* Boucle sur tous les elements de la liste */
	while (elementItem != NULL)
	{
		sprintf(buffer,"%-55s ", elementItem->element.description);
		write(id,buffer,strlen(buffer));

		if (elementItem->element.d.ot == NULL)
		{
			sprintf(buffer,"Data on disk\n");
			write(id,buffer,strlen(buffer));
		}
		else
		{
			switch (elementItem->element.representation)
			{
			case DCM_AE:
			case DCM_AS:
			case DCM_CS:
			case DCM_DA: 	stringLength=MIN(sizeof(scratch)-1,elementItem->element.length);
					strncpy(scratch,elementItem->element.d.string,stringLength);
					scratch[stringLength] = '\0';
					write(id,scratch, strlen(scratch));
					write(id,"\n",1);
					break;
			case DCM_DD:
			case DCM_FD:
			case DCM_FL:	sprintf(buffer,"Unimplemented\n");
					write(id,buffer,strlen(buffer));
					break;
			case DCM_DS:
			case DCM_IS:
			case DCM_LO:
			case DCM_LT:
			case DCM_PN:
			case DCM_SH:	stringLength=MIN(sizeof(scratch)-1,elementItem->element.length);
					strncpy(scratch,elementItem->element.d.string,stringLength);
					scratch[stringLength] = '\0';
					write(id,scratch, strlen(scratch));
					write(id,"\n",1);
					break;
			case DCM_SL:	sprintf(buffer, "%8x %d\n", *elementItem->element.d.sl,*elementItem->element.d.sl);
					write(id,buffer,strlen(buffer));
					break;
			case DCM_SQ:	sprintf(buffer,"SEQUENCE\n");
					write(id,buffer,strlen(buffer));
					break;
			case DCM_ST:	stringLength=MIN(sizeof(scratch)-1,elementItem->element.length);
					strncpy(scratch,elementItem->element.d.string,stringLength);
					scratch[stringLength] = '\0';
					write(id,scratch, strlen(scratch));
					write(id,"\n",1);
					break;
			case DCM_TM:
			case DCM_UI:	stringLength=MIN(sizeof(scratch)-1,elementItem->element.length);
					strncpy(scratch,elementItem->element.d.string,stringLength);
					scratch[stringLength] = '\0';
					write(id,scratch, strlen(scratch));
					write(id,"\n",1);
					break;
			case DCM_AT:
			case DCM_UL:	sprintf(buffer,"%8x %d\n",*elementItem->element.d.ul,*elementItem->element.d.ul);
					write(id,buffer,strlen(buffer));
					break;
			case DCM_US:	sprintf(buffer,"%4x %d\n",*elementItem->element.d.ul,*elementItem->element.d.ul);
					write(id,buffer,strlen(buffer));
					break;
			case DCM_OT:
			case DCM_OW:
			case DCM_OB:
			case DCM_UNKNOWN:
			case DCM_RET:	sprintf(buffer,"Unimplemented\n");
					write(id,buffer,strlen(buffer));
					break;
			default:	sprintf(buffer,"unknown representation type\n");
					write(id,buffer,strlen(buffer));
					break;
			}
		}
		elementItem = LST_Next(&elementList);
	}
	return(0);
}


