/******************************************************************************/
/* Functions used to dilate an image from Scilab                              */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include "int_imdilate.h"

/******************************************************************************/
/* Interface Scilab <-> imdilate                                              */
/******************************************************************************/
/* Creation : 06/08/2001 (V. COUVERT)                                         */
/* Last modification : 23/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imdilate(char *fname)
{
	int l_image, m_image, n_image,
	    l_kernel, m_kernel, n_kernel,
	    m_dilated_image, n_dilated_image,
	    l_mask, m_mask, n_mask;
	double *l_dilated_image;

	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Ouput arguments */

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "d", &m_kernel, &n_kernel, &l_kernel);
	GetRhsVar(3, "d", &m_mask, &n_mask, &l_mask);
	
	dilate_image(stk(l_image), m_image, n_image, stk(l_kernel), m_kernel, n_kernel, &l_dilated_image, stk(l_mask), &ierr);
	
	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(4,"d", &m_image, &n_image, &l_dilated_image);

	if(l_dilated_image) {
		free(l_dilated_image);
	}

	LhsVar(1)=4;

	return(0);
}

/******************************************************************************/
/* Image dilation function                                                    */
/******************************************************************************/
/* Creation : 06/08/2001 (V. COUVERT)                                         */
/* Last modification : 18/04/2002 (V. COUVERT)                                */
/******************************************************************************/
int dilate_image(double *image, int width, int height, double *kernel,
				 int kernel_width, int kernel_height, double **dilated_image, 
				 double *mask, int *ierr)
{
	static int x, y, xscrut, yscrut;
	double max=0.0;

	/* Memory allocation */
	if (!(*dilated_image = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}

	/* Filling eroded image */
	for(y=0 ; y<height ; y++)
	{
		for(x=0 ; x<width ; x++)
		{
			/* If the pixel is not masked */
			if(*(mask+x+y*width)!=0)
			{
				/* Find the minimum */
				for(yscrut=-kernel_height/2 ; yscrut<=kernel_height/2 ; yscrut++)
				{
					for(xscrut=-kernel_width/2 ; xscrut<=kernel_width/2 ; xscrut++)
					{
						/* If value has to be count : kernel corresponding value is != 0*/
						if(kernel[xscrut+kernel_width/2+(yscrut+kernel_height/2)*kernel_width]!=0)
						{
							/* If we point a pixel in the image (to take border in account) and the pixel is not masked*/
							if( (x+xscrut>=0) && (x+xscrut<width) && (y+yscrut>=0) && (y+yscrut<height) && ((*(mask+x+xscrut+(y+yscrut)*width))!=0) )
							{
								if(*(image+x+y*width+xscrut+yscrut*width)>max)
								{
									max=*(image+x+y*width+xscrut+yscrut*width);
								}
							}
						}
					}
				}
				(*dilated_image)[x+y*width]=max;
				max=0.0;
			}
			else
			{
				(*dilated_image)[x+y*width]=*(image+x+y*width);
			}
		}
	}
	*ierr=0;
	return(0);
}

