
/******************************************************************************/
/* Fonctions used to carry out the fuzzy k-means method classification phase  */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include <fuzzykmeanslib.h>
#include "int_imfuzzykmeansclassify.h"

/******************************************************************************/
/* Interface Scilab<->imfuzzykmeansclassify_2                                 */
/******************************************************************************/
/* Creation : 01/04/2004 (V. POMMEREUL)                                       */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/

int int_imfuzzykmeansclassify_2(char *fname) {

	// Input and output arguments
	static int l_dimension, m_dimension, n_dimension,
			   l_prototype, m_prototype, n_prototype,
			   l_mask, m_mask, n_mask,
			   l_weights, m_weights, n_weights,
			   l_fuzzyness, m_fuzzyness, n_fuzzyness,
			   l_threshold1, m_threshold1, n_threshold1,
			   l_threshold2, m_threshold2, n_threshold2,
			   m_image, n_image,
			   m_classes_image, n_classes_image;
	static int l_images[NBIMAGES_MAX];
	double *l_classes_image;
	HyperMat h_proto, h_mask, h_images[NBIMAGES_MAX], h_classes_image;

	// Number of input and output arguments
	int minlhs = 1, maxlhs = 1;
	int minrhs = 8, maxrhs = 17;

	int ierr=0;

	int ind_var=1;
	int ind_res=1;

	int dim, nbIm;
	double *s_mask, *s_images[NBIMAGES_MAX], *proto, *weights;
	double fuzzyness, threshold1, threshold2;

	int i;


	// Verify number of arguments
	CheckRhs(minrhs, maxrhs); // Input arguments
	CheckLhs(minlhs, maxlhs); // Output arguments

	// Get right hand side arguments

	// Get dimension of image
	GetRhsVar(ind_var++, "i", &m_dimension, &n_dimension, &l_dimension);
	dim = *istk(l_dimension);

	// Get classes prototype
	if (dim == 2 | dim == 3) {
		// The prototype is a matrix
		GetRhsVar(ind_var++, "d", &m_prototype, &n_prototype, &l_prototype);
		proto = stk(l_prototype);
		nbIm = m_prototype;

	} else {
		// The prototype is an hypermatrix
		GetRhsHMat(ind_var++, &h_proto);
		nbIm = h_proto.dims[0];
	}

	// Get the image mask
	if (dim == 2) {
		// The mask is a matrix
		GetRhsVar(ind_var++, "d", &m_mask, &n_mask, &l_mask);
	} else {
		// The mask is an hypermatrix
		GetRhsHMat(ind_var++, &h_mask);
	}

	// Get the weights associated to each image
	GetRhsVar(ind_var++, "d", &m_weights, &n_weights, &l_weights);
	weights = stk(l_weights);

	// Get the fuzzyness
	GetRhsVar(ind_var++, "d", &m_fuzzyness, &n_fuzzyness, &l_fuzzyness);
	fuzzyness = *stk(l_fuzzyness);

	// Get the thresholds
	GetRhsVar(ind_var++, "d", &m_threshold1, &n_threshold1, &l_threshold1);
	threshold1 = *stk(l_threshold1);
	GetRhsVar(ind_var++, "d", &m_threshold2, &n_threshold2, &l_threshold2);
	threshold2= *stk(l_threshold2);


	// Get data images
	for (i=0 ; i<nbIm ; i++) {
		if (dim == 2) {
			// The images are matrix
			GetRhsVar(ind_var++, "d", &m_image, &n_image, &l_images[i]);
		} else {
			// The images are hypermatrix
			GetRhsHMat(ind_var++, &h_images[i]);
		}
	}

	// Compute fuzzy k-means method classification phase
	if (dim == 2) {
		s_mask = stk(l_mask);

		for (i=0 ; i<nbIm ; i++) {
			s_images[i] = stk(l_images[i]);
		}
		// 2D images
		ierr = classify_fuzzykmeans_2D(proto, m_prototype, n_prototype, s_mask, weights, fuzzyness, threshold1,
									threshold2, s_images, m_image, n_image, &l_classes_image);

	} else if (dim == 3){
		// 3D images
		ierr = classify_fuzzykmeans_3D(proto, m_prototype, n_prototype, h_mask, weights, fuzzyness, threshold1,
									threshold2, h_images, &h_classes_image);
	} else {
		// 4D images
		ierr = classify_fuzzykmeans_4D(h_proto, h_mask, weights, fuzzyness, threshold1, threshold2, h_images, 												&h_classes_image);
	}


	// In case of error
	if(ierr!=0) {
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	// Return classes image
	if (dim == 2) {
		CreateVarFromPtr(ind_var, "d", &m_image, &n_image, &l_classes_image);
		if(l_classes_image) {
			free(l_classes_image);
		}
		LhsVar(ind_res++)=ind_var;

	} else {

		CreateHMat(ind_var,&h_classes_image);
		LhsVar(ind_res++) = ind_var;
		// Memory desallocation
		if (h_classes_image.dims) {
			free(h_classes_image.dims);
		}
		if (h_classes_image.R) {
			free(h_classes_image.R);
		}
	}

	return(0);

}


