/*********************************************************************************************************************/
/*                        Function used for region growing in an image from seeds points                             */
/*                                                                                                                   */
/*********************************************************************************************************************/
//#include "/usr/lib/scilab-2.7/contrib/vivilab/src/int_imgrowing.h"

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include <math.h>

#include "int_imgrowing.h"

/*********************************************************************************************************************/
/* Interface Scilab <> imgrowing                                                                                     */
/*********************************************************************************************************************/
/* Creation : 25/03/2004	                                                                                     */
/* Last modification : 11/05/2004                                                                                    */
/* Author : Virigna SOUKPHOUANGKHAM                                                                                  */
/*********************************************************************************************************************/
int int_imgrowing(char *fname)
{
	int l_image_treated, m_image_treated, n_image_treated,
		l_mask_treated, m_mask_treated, n_mask_treated,
		l_points, m_points, n_points,
		l_threshold, m_threshold, n_threshold,
		l_threshold2, m_threshold2, n_threshold2,
		l_size_seed, m_size_seed, n_size_seed,
		l_choice, m_choice, n_choice,
		*l_im_regions, m_im_regions, n_im_regions,
		m_iter, n_iter;
	double *l_iter;

	int ierr=0;

	int minlhs=1, maxlhs=2,
		minrhs=7, maxrhs=7;

	CheckRhs(minrhs,maxrhs);	//input arg
	CheckLhs(minlhs,maxlhs);	//output arg

	GetRhsVar(1, "d", &m_image_treated, &n_image_treated, &l_image_treated);
	GetRhsVar(2, "d", &m_mask_treated, &n_mask_treated, &l_mask_treated);
	GetRhsVar(3, "i", &m_points, &n_points, &l_points);
	GetRhsVar(4, "d", &m_threshold, &n_threshold, &l_threshold);
	GetRhsVar(5, "d", &m_threshold2, &n_threshold2, &l_threshold2);
	GetRhsVar(6, "i", &m_size_seed, &n_size_seed, &l_size_seed);
	GetRhsVar(7, "i", &m_choice, &n_choice, &l_choice);

	growing_region(stk(l_image_treated), stk(l_mask_treated), m_image_treated, 
		n_image_treated, istk(l_points), n_points, *istk(l_size_seed), *stk(l_threshold), 
		*stk(l_threshold2), *istk(l_choice), &l_im_regions, &l_iter, &ierr);

	//In case of error
	if (ierr>0)
		{
		Scierror(999,"%s :Internal error\r\n",fname);
		}

	//Sizes of output variables
	m_im_regions	= m_image_treated;
	n_im_regions	= n_image_treated;
	m_iter		= 1;
	n_iter		= n_points;

	CreateVarFromPtr(8,"d", &m_im_regions, &n_im_regions, &l_im_regions);
	CreateVarFromPtr(9,"i", &m_iter, &n_iter, &l_iter);

	if (l_im_regions) {
		free(l_im_regions);
	}
	if (l_iter) {
		free(l_iter);
	}

	LhsVar(1)=8;
	LhsVar(2)=9;

	return(0);
}


/*********************************************************************************************************************/
/* Region expansion function                                                                                         */
/*********************************************************************************************************************/
/* Creation : 25/03/2004                                                                                             */
/* Last modification : 11/05/2004                                                                                    */
/* Author : Virigna SOUKPHOUANGKHAM                                                                                  */
/*********************************************************************************************************************/
int growing_region(double *image, double *mask, int width, int height, int *seeds, int seeds_nb, int seeds_size, double thres, double thres2, int choix, double **mask_region, int **iter, int *ierr)
{
	int *region;		//contains the coordinates of the elts of the region
	int *tmp_region;	//region associated with one seed
	int cpt=0, tmp_cpt=0;	//counters for region and tmp_region, indicate current elt
	int *candidates;	//contains the candidates for the agregation
	int n=0;		//nb of elements in candidates
	int test,test1,test2;
	int *last_added;	//contains the pixels added at the last iteration
	int m=0;		//nb of elts agregated at the last iteration
	int removed;		//nb of elts added to the region
	int i=0,j=0,k=0,p=0,	//loop counters ( i,j : region
		q=0;		//		  k : candidates)
	int x=0,y=0;		//for the pixels tested (= -1, 0 or 1)
	int xs=0,ys=0;		//seeds size
	double mean,mean_region;//used for the criteria of agregation
	double var=0;		//variance of the region
	double dev;		//deviance of the region

	//memory allocation
	//-----------------
	if (!((*mask_region)	= (double*)calloc(1,sizeof(double)*width*height)))
		{
		*ierr=1;
		return(*ierr);
		}
	if (!((region)		= (int*)calloc(1,sizeof(int)*2*width*height)))
		{
		*ierr=1;
		return(*ierr);
		}
	if (!((tmp_region)	= (int*)calloc(1,sizeof(int)*2*width*height)))
		{
		*ierr=1;
		return(*ierr);
		}
	if (!((candidates)	= (int*)calloc(1,sizeof(int)*2*width*height)))
		{
		*ierr=1;
		return(*ierr);
		}
	if (!((last_added)	= (int*)calloc(1,sizeof(int)*2*width*height)))
		{
		*ierr=1;
		return(*ierr);
		}

	//initialization
	//--------------
	cpt		= 0;
	region[0]	= -1;
	region[1]	= -1;
	xs		= floor(seeds_size/2);
	ys		= xs;

	//choice for seeds
	switch(choix)
	{
	///////////////////////////////////////////
	case 1:	 	// each seed is a region //
	///////////////////////////////////////////
	if (!((*iter)		= (int*)calloc(1,sizeof(int)*seeds_nb)))
		{
		*ierr=1;
		return(*ierr);
		}

	for(q=0;q<seeds_nb;q++)	//loop on the seeds (main loop)
		{
		mean_region	= 0;
		mean	= 0;
		tmp_cpt	= 0;
		var	= 0;
		dev	= 0;
		test	= 0;
		for(x=-xs;x<=xs;x++)
			{
			for(y=-ys;y<=ys;y++)
				{
				for(i=0;i<=cpt;i++)
					{
					if((region[2*i]==*(seeds+2*q)+x) && (region[2*i+1]==*(seeds+2*q+1)+y))	{test=1;break;}	//if the seed is already in region
					}
					if(test==0 && ((*(mask+(*(seeds+2*q)+x)-1+((*(seeds+2*q+1)+y)-1)*width)) != 1))
						{
						tmp_region[2*tmp_cpt]	= *(seeds+2*q)+x;
						tmp_region[2*tmp_cpt+1]	= *(seeds+2*q+1)+y;
						region[2*cpt]		= *(seeds+2*q)+x;	//add the seed in region
						region[2*cpt+1]		= *(seeds+2*q+1)+y;
						mean 			= (mean*tmp_cpt+*(image+tmp_region[2*tmp_cpt]-1+((tmp_region[2*tmp_cpt+1]-1)*width)))/(tmp_cpt+1);
						mean_region		= (mean_region*tmp_cpt+(*(image+tmp_region[2*tmp_cpt]-1+(tmp_region[2*tmp_cpt+1]-1)*width)))/(tmp_cpt+1);
						cpt++;
						tmp_cpt++;
						}
				}
			}
		var	= 0;
		for(i=0;i<tmp_cpt;i++)
			{
			var	= var + ((*(image+tmp_region[2*i]-1+(tmp_region[2*i+1]-1)*width)-mean_region)*(*(image+tmp_region[2*i]-1+(tmp_region[2*i+1]-1)*width)-mean_region))/tmp_cpt;
			}
		dev	= sqrt(var);
		removed	= 1;

	//-------------------------------------------------------------------------------------------------------------------//
	// Region expansion                                                                                                  //
	//-------------------------------------------------------------------------------------------------------------------//
	(*iter)[q]= 0;
	while(removed != 0  && (*iter)[q]<100)
		{
		candidates[0]=-1;
		candidates[1]=-1;

		//neighbourhood creation (in 8-connectivity)
		n=0;		//counter for candidates
		for(i=0;i<tmp_cpt;i++)	//loop on the elts of tmp_region
			{
			for(y=-1;y<=1;y++)	//loops on the neighbour pixels
				{
				for(x=-1;x<=1;x++)
					{
					test1=0;
					test2=0;
					if (((tmp_region[2*i]+x)<=width) && ((tmp_region[2*i+1]+y)<=height) && ((tmp_region[2*i]+x)>0) && ((tmp_region[2*i+1]+y)>0) && (x!=0 || y!=0) && ((*(mask+(tmp_region[2*i]+x-1)+(tmp_region[2*i+1]+y-1)*width)) != 1))	//if the pixel tested is not himself and is not in the mask considered, and indexes don't exceed
						{
						for(j=0;j<=tmp_cpt;j++)
							{
							if((tmp_region[2*j] == (tmp_region[2*i]+x)) && (tmp_region[2*j+1] == (tmp_region[2*i+1]+y)))		{test1=1;break;}	//if the pixel tested is in region set test1 to 1
							}
						if(test1==0)	//if the pixel is not in region
							{
							for(k=0;k<n;k++)
								{
								if((candidates[2*k] == (tmp_region[2*i]+x)) && (candidates[2*k+1] == (tmp_region[2*i+1]+y)))		{test2=1;break;}	//if the pixel is already in candidates set test2 to 1
								}
								if(test2==0)	//if the pixel is not in candidates
									{
									//entering coordinates of the pixel into candidates
									candidates[2*n]   = tmp_region[2*i]+x;
									candidates[2*n+1] = tmp_region[2*i+1]+y;
									n++;
									}
							}
						}
					}
				}
			}
		//set to zero
		removed	= 0;
		m	= 0;	//counter for last added

		//loop on the elts of candidates
		for(i=0;i<n;i++)	//n : nb of elts in candidates
			{
			if ((distance(*(image+candidates[2*i]-1+(candidates[2*i+1]-1)*width),mean)<thres) && (distance(distance(*(image+candidates[2*i]-1+(candidates[2*i+1]-1)*width),mean_region),dev)<thres2))
				{
				removed++;
				//updating region, tmp_region and last_added
				tmp_region[2*tmp_cpt]   = candidates[2*i];
				tmp_region[2*tmp_cpt+1] = candidates[2*i+1];
				region[2*cpt]   	= candidates[2*i];
				region[2*cpt+1] 	= candidates[2*i+1];
				mean_region		= (mean_region*tmp_cpt+(*(image+candidates[2*i]-1+((candidates[2*i+1]-1)*width))))/(tmp_cpt+1);
				last_added[2*m]		= candidates[2*i];
				last_added[2*m+1]	= candidates[2*i+1];
				m++;
				cpt++;	//1 more elt in region
				tmp_cpt++;
				}
			}
			var	= 0;
			for(i=0;i<tmp_cpt;i++)
				{
				var	= var + ((*(image+tmp_region[2*i]-1+(tmp_region[2*i+1]-1)*width)-mean_region)*(*(image+tmp_region[2*i]-1+(tmp_region[2*i+1]-1)*width)-mean_region))/tmp_cpt;
				}
			dev	= sqrt(var);

		//re-initialization (mean of the agregated pixels)
		mean = 0;
		for(i=0;i<m;i++)	//m : number of agregated pixels
			{
			mean = mean+*(image+last_added[2*i]-1+((last_added[2*i+1]-1)*width));
			}
		mean = mean/m;
		(*iter)[q]++;
		}	//end of while loop (expansion)
	//---------------------------------------------------------------------------------------------------------------------
		}	//end of for (finishing describing all the seeds)
		break;


	//////////////////////////////////////////////////
	case 2:		//all seeds are the same region //
	//////////////////////////////////////////////////
	if (!((*iter)		= (int*)calloc(1,sizeof(int))))
		{
		*ierr=1;
		return(*ierr);
		}
	for(q=0;q<seeds_nb;q++)	//loop on the seeds (main loop)
		{
		mean_region	= 0;
		var	= 0;
		dev	= 0;
		test	= 0;
		if(test==0 && ((*(mask+(*(seeds+2*q))-1+((*(seeds+2*q+1))-1)*width)) != 1))
			{
			region[2*cpt]		= *(seeds+2*q);	//add the seed in region
			region[2*cpt+1]		= *(seeds+2*q+1);
			mean 			= (mean*cpt+*(image+region[2*cpt]-1+((region[2*cpt+1]-1)*width)))/(cpt+1);
			mean_region		= (mean_region*cpt+(*(image+region[2*cpt]-1+(region[2*cpt+1]-1)*width)))/(cpt+1);
			cpt++;
			}
		}
	var	= 0;
	for(i=0;i<cpt;i++)
		{
		var	= var + ((*(image+region[2*i]-1+(region[2*i+1]-1)*width)-mean_region)*(*(image+region[2*i]-1+(region[2*i+1]-1)*width)-mean_region))/cpt;
		}
	dev	= sqrt(var);

	removed	= 1;
	//-------------------------------------------------------------------------------------------------------------------//
	// region expansion
	(*iter)[0]	= 0;
	while(removed != 0  && (*iter)[0]<100)
		{
		candidates[0]=-1;
		candidates[1]=-1;

		//neighbourhood creation (in 8-connectivity)
		n=0;		//counter for candidates
		for(i=0;i<cpt;i++)	//loop on the elts of tmp_region
			{
			for(y=-1;y<=1;y++)	//loops on the neighbour pixels
				{
				for(x=-1;x<=1;x++)
					{
					test1=0;
					test2=0;
					if (((region[2*i]+x)<=width) && ((region[2*i+1]+y)<=height) && ((region[2*i]+x)>0) && ((region[2*i+1]+y)>0) && (x!=0 || y!=0) && ((*(mask+(region[2*i]+x-1)+(region[2*i+1]+y-1)*width)) != 1))	//if the pixel tested is not himself and is not in the mask considered, and indexes don't exceed
						{
						for(j=0;j<=cpt;j++)
							{
							if((region[2*j] == (region[2*i]+x)) && (region[2*j+1] == (region[2*i+1]+y)))		{test1=1;break;}	//if the pixel tested is in region set test1 to 1
							}
						if(test1==0)	//if the pixel is not in region
							{
							for(k=0;k<n;k++)
								{
								if((candidates[2*k] == (region[2*i]+x)) && (candidates[2*k+1] == (region[2*i+1]+y)))		{test2=1;break;}	//if the pixel is already in candidates set test2 to 1
								}
								if(test2==0)	//if the pixel is not in candidates
									{
									//entering coordinates of the pixel into candidates
									candidates[2*n]   = region[2*i]+x;
									candidates[2*n+1] = region[2*i+1]+y;
									n++;
									}
							}
						}
					}
				}
			}
		//set to zero
		removed	= 0;
		m	= 0;	//counter for last added

		//loop on the elts of candidates
		for(i=0;i<n;i++)	//n : nb of elts in candidates
			{
			if ((distance(*(image+candidates[2*i]-1+(candidates[2*i+1]-1)*width),mean)<thres) && (distance(distance(*(image+candidates[2*i]-1+(candidates[2*i+1]-1)*width),mean_region),dev)<thres2))
				{
				removed++;
				//updating region, tmp_region and last_added
				region[2*cpt]   	= candidates[2*i];
				region[2*cpt+1] 	= candidates[2*i+1];
				mean_region		= (mean_region*cpt+(*(image+candidates[2*i]-1+((candidates[2*i+1]-1)*width))))/(cpt+1);
				last_added[2*m]		= candidates[2*i];
				last_added[2*m+1]	= candidates[2*i+1];
				m++;
				cpt++;	//1 more elt in region
				}
			}

		//updating deviation
		var	= 0;
		for(i=0;i<cpt;i++)
			{
			var	= var + ((*(image+region[2*i]-1+(region[2*i+1]-1)*width)-mean_region)*(*(image+region[2*i]-1+(region[2*i+1]-1)*width)-mean_region))/cpt;
			}
		dev	= sqrt(var);

		//re-initialization (mean of the agregated pixels)
		mean = 0;
		for(i=0;i<m;i++)	//m : number of agregated pixels
			{
			mean = mean+*(image+last_added[2*i]-1+((last_added[2*i+1]-1)*width));
			}
		mean = mean/m;
		(*iter)[0]++;
		}	//end of while loop (expansion)*/
	//-------------------------------------------------------------------------------------------------------------------//
		break;
	}






	//creation of the region mask
	//---------------------------
	for(i=0;i<width*height;i++)
		{
		(*mask_region)[i]=0;
		}
	for(i=0;i<cpt;i++)
		{
		(*mask_region)[region[2*i]-1+(region[2*i+1]-1)*width]=1;
		}

	//end of function
	//---------------
	free(region);
	free(tmp_region);
	free(candidates);
	free(last_added);

	*ierr=0;
	return(0);
}

/*********************************************************************************************************************/
/* Distance definition function                                                                                      */
/*********************************************************************************************************************/
double distance(double a, double b)
{
	double d;
	d=abs(a-b);
	return(d);
}
/******************************************end of file****************************************************************/
