/******************************************************************************/
/* Functions used to compute an image histogram from Scilab                   */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include <math.h>
#include "int_imhist.h"

/******************************************************************************/
/* Interface Scilab <-> imhist                                                */
/******************************************************************************/
/* Creation : 13/07/2001 (V. COUVERT)                                         */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imhist(char *fname)
{
	int l_image, m_image, n_image,
	    m_histo, n_histo=1,
	    l_step, m_step, n_step,
	    l_mask, m_mask, n_mask,
	    l_min, m_min, n_min,
	    l_max, m_max, n_max;
	double *l_histo;
	
	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=5, maxrhs=5;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Ouput arguments */

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "d", &m_step, &n_step, &l_step);
	GetRhsVar(3, "d", &m_mask, &n_mask, &l_mask);
	GetRhsVar(4, "d", &m_min, &n_min, &l_min);
	GetRhsVar(5, "d", &m_max, &n_max, &l_max);

	hist(stk(l_image), m_image, n_image, &l_histo, &m_histo, *stk(l_step), stk(l_mask), *stk(l_min), *stk(l_max), &ierr);

	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(6,"i", &n_histo, &m_histo, &l_histo);

	if (l_histo) {
		free(l_histo);
	}

	LhsVar(1)=6;

	return(0);
}


/******************************************************************************/
/* Histogram computing                                                        */
/******************************************************************************/
/* Creation : 18/07/2001 (V. COUVERT)                                         */
/* Last modification : 02/05/2002 (V. COUVERT)                                */
/******************************************************************************/
int hist(double *image, int width, int height, int **histo, int *hist_size, 
		 double step, double *mask, double min, double max, int *ierr)
{
	int x, y, i, val;

	*hist_size=ceil( (max-min)/step ); //round by excesses

	/* Memory allocation */
	if (!((*histo) = (int*)calloc(1, sizeof(int)*(*hist_size))))
	{
		*ierr=1;
		return(-1);
	}

	/* Histogram computing */
	for(y = 0 ; y < height ; y++)
	{
		for(x = 0 ; x < width ; x++)
		{
			if(*(mask+x+y*width)!=0.0)
			{
				if( (*(image+x+y*width)>=min) && (*(image+x+y*width)<=max))
				{
					val=floor( (*(image+x+y*width)-min) /step );
					(*histo)[val]++;
				}
			}
		}
	}

	*ierr=0;
	return(0);
}
