/******************************************************************************/
/* Functions used to compute the histogram of a color image from Scilab       */
/******************************************************************************/
#include <math.h>
#include "stack-c.h"
#include "int_imhist3D.h"

/******************************************************************************/
/* Interface Scilab <-> imhist3D                                              */
/******************************************************************************/
/* Creation : 30/04/2002 (S. BONDU)                                           */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imhist3D(char *fname)
{	
	int l_image1, m_image1, n_image1, // L*, x, R
	    l_image2, m_image2, n_image2, // a*, y, V 
	    l_image3, m_image3, n_image3, // b*, z, B 
	    l_step, m_step, n_step,
	    l_mask, m_mask, n_mask,
	    l_min1, m_min1, n_min1,
	    l_min2, m_min2, n_min2,
	    l_min3, m_min3, n_min3,
	    l_max1, m_max1, n_max1,
	    l_max2, m_max2, n_max2,
	    l_max3, m_max3, n_max3,
	    m_hist, n_hist=1;
	double *l_hist;
	int ierr=0;
    	
	int minlhs=1, maxlhs=1, minrhs=11, maxrhs=11;

	CheckRhs(minrhs, maxrhs); // Input arguments 
	CheckLhs(minlhs, maxlhs); // Output arguments
	
	GetRhsVar(1, "d", &m_image1, &n_image1, &l_image1);
	GetRhsVar(2, "d", &m_image2, &n_image2, &l_image2);
	GetRhsVar(3, "d", &m_image3, &n_image3, &l_image3);
	GetRhsVar(4, "d", &m_step, &n_step, &l_step);
	GetRhsVar(5, "d", &m_mask, &n_mask, &l_mask);
	GetRhsVar(6, "d", &m_min1, &n_min1, &l_min1);
	GetRhsVar(7, "d", &m_max1, &n_max1, &l_max1);
	GetRhsVar(8, "d", &m_min2, &n_min2, &l_min2);
	GetRhsVar(9, "d", &m_max2, &n_max2, &l_max2);
	GetRhsVar(10, "d", &m_min3, &n_min3, &l_min3);
	GetRhsVar(11, "d", &m_max3, &n_max3, &l_max3);

	hist3D(stk(l_image1), stk(l_image2), stk(l_image3), stk(l_mask), 
		m_image1, n_image1, 
		*stk(l_min1), *stk(l_max1), 
		*stk(l_min2), *stk(l_max2), 
		*stk(l_min3), *stk(l_max3), 
		&l_hist, &m_hist, *stk(l_step), &ierr);
		

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(12,"d", &m_hist, &n_hist, &l_hist);

	if(l_hist) {
		free(l_hist);
	}

	LhsVar(1)=12;

	return(0);
}

/******************************************************************************/
/* Histogram computing function                                               */
/******************************************************************************/
/* Creation : 30/04/2002 (S. BONDU)                                           */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int hist3D(double *p1, double *p2, double *p3, double *pMask, 
		   int Width, int Height, double min1, double max1, double min2, 
		   double max2, double min3, double max3, double **pHist3D, 
		   int *hist_size, double step, int *ierr)
{
	int y, x;
	int val1, val2, val3;

	*hist_size=(int) (ceil((max1-min1)/step)
					 *ceil((max2-min2)/step)
					 *ceil((max3-min3)/step));

	// Memory allocation 
	if (!(*pHist3D = (double*)calloc((*hist_size), sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}

	for(y=0;y<Height;y++)
	{
		for(x=0;x<Width;x++)
		{
			if(pMask[x+y*Width] != 0.0)
			{
				if( ((p1[x+y*Width]>=min1) && (p1[x+y*Width]<=max1)) &&
				    ((p2[x+y*Width]>=min2) && (p2[x+y*Width]<=max2)) &&
				    ((p3[x+y*Width]>=min3) && (p3[x+y*Width]<=max3)) )
				{
					val1 = (int)floor(( p1[x+y*Width] - min1)/step);
					val2 = (int)floor(( p2[x+y*Width] - min2)/step);
					val3 = (int)floor(( p3[x+y*Width] - min3)/step);

					(*pHist3D)[(val1 * ( (int)(floor((max2-min2)/step))) + val2) * 
						( (int)(floor((max3-min3)/step))) + val3]++;
				}
			}
		}
	}

	*ierr = 0;
	return (0);
}
