
/******************************************************************************/
/* Fonctions used to create learning data used in training phases             */
/******************************************************************************/


#include "int_imlearningdata.h"


/******************************************************************************/
/* Interface Scilab<->imlearningdata                                          */
/******************************************************************************/
/* Creation : 19/04/2004 (V. POMMEREUL)                                       */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imlearningdata(char *fname) {

	// Input and output arguments
	static int 	l_dimension, m_dimension, n_dimension,
				l_imnb, m_imnb, n_imnb,
				l_pointsnb, m_pointsnb, n_pointsnb,
				l_mask, m_mask, n_mask,
				l_weights, m_weights, n_weights,
				m_image, n_image;
	static int l_images[NBIMAGES_MAX];
	double *l_learningdata;
	HyperMat h_mask, h_images[NBIMAGES_MAX], h_learningdata;

	// Number of input and output arguments
	int minlhs = 1, maxlhs = 1;
	int minrhs = 6, maxrhs = 15;

	int ierr=0;

	int ind_var=1;
	int ind_res=1;

	int dim, nbIm, pointsNb;
	double *s_mask, *s_images[NBIMAGES_MAX], *weights;

	int i;

	// Verify number of arguments
	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */

	// Get right hand side arguments
	GetRhsVar(ind_var++, "i", &m_dimension, &n_dimension, &l_dimension);
	dim = *istk(l_dimension);
	GetRhsVar(ind_var++, "i", &m_imnb, &n_imnb, &l_imnb);
	nbIm = *istk(l_imnb);
	GetRhsVar(ind_var++, "i", &m_pointsnb, &n_pointsnb, &l_pointsnb);
	pointsNb = *istk(l_pointsnb);

	if (dim == 2) {
		GetRhsVar(ind_var++, "d", &m_mask, &n_mask, &l_mask);
		s_mask = stk(l_mask);

	} else {
		GetRhsHMat(ind_var++, &h_mask);
	}

	GetRhsVar(ind_var++, "d", &m_weights, &n_weights, &l_weights);
	weights = stk(l_weights);

	for (i=0 ; i<nbIm ; i++) {
		if (dim == 2) {
			GetRhsVar(ind_var++, "d", &m_image, &n_image, &l_images[i]);
		} else {
			GetRhsHMat(ind_var++, &h_images[i]);
		}
	}

	// Compute learning data creation
	if (dim == 2) {

		for (i=0 ; i<nbIm ; i++) {
			s_images[i] = stk(l_images[i]);
		}

		ierr = learningdata_2D(nbIm, pointsNb, s_mask, weights, s_images, m_image, n_image, &l_learningdata);

	} else if (dim == 3){
		ierr = learningdata_3D(nbIm, pointsNb, h_mask, weights, h_images, &l_learningdata);

	} else {
		ierr = learningdata_4D(nbIm, pointsNb, h_mask, weights, h_images, &h_learningdata);
	}


	// In case of error
	if(ierr!=0) {
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	// Return classes image
	if (dim == 2 || dim == 3) {
		CreateVarFromPtr(ind_var, "d", &pointsNb, &nbIm, &l_learningdata);
		
		if (l_learningdata) {
			free(l_learningdata);
		}

		LhsVar(ind_res)=ind_var;

	} else {

		CreateHMat(ind_var,&h_learningdata);
		LhsVar(ind_res) = ind_var;
		// Memory desallocation
		if (h_learningdata.dims) {
			free(h_learningdata.dims);
		}
		if (h_learningdata.R) {
			free(h_learningdata.R);
		}
	}

	return(0);

}


/******************************************************************************/
/* Create learning data from 2D images                                        */
/******************************************************************************/
int learningdata_2D(int nbIm, int pointsNb, double *mask, double *weights,
					double *images[NBIMAGES_MAX], int height, int width, double **learningData){

	int k;
	int err = 0;
	const int nbDim = 3;
	const int nbComp = 1;
	int dimvar = 0;
	HyperMat hmask, hld, himages[NBIMAGES_MAX];


	// ........................................................................
	// Create an hypermatrix array for images
	for (k=0 ; k<nbIm ; k++) {
		himages[k].dimsize = nbDim;
		dimvar = 0;
		if (himages[k].dims = (int *) calloc(himages[k].dimsize, sizeof(int))) {
			himages[k].dims[dimvar++] = height;
			himages[k].dims[dimvar++] = width;
			himages[k].dims[dimvar++] = nbComp;
		} else {
			return 1;
		}
		himages[k].size = height * width * nbComp;
		himages[k].R = images[k];
		himages[k].it = 0;
	}


	// ........................................................................
	// Create an hypermatrix for the mask
	hmask.dimsize = nbDim;
	dimvar = 0;
	if (hmask.dims = (int *) calloc(hmask.dimsize, sizeof(int))) {
		hmask.dims[dimvar++] = height;
		hmask.dims[dimvar++] = width;
		hmask.dims[dimvar++] = nbComp;
	} else {
		return 1;
	}
	hmask.size = height * width * nbComp;
	hmask.R = mask;
	hmask.it = 0;

	// ........................................................................
	// Memory allocation for learning data
	if (*learningData = (double*) calloc(pointsNb * nbIm * nbComp, sizeof(double))) {
		// Create an hypermatrix for learning data
		hld.dimsize = nbDim;
		dimvar = 0;
		if (hld.dims = (int *) calloc(hld.dimsize, sizeof(int))) {
			hld.dims[dimvar++] = pointsNb;
			hld.dims[dimvar++] = nbIm;
			hld.dims[dimvar++] = nbComp;
		} else {
			return 1;
		}
		hld.size = pointsNb * nbIm * nbComp;
		hld.R = (*learningData);
		hld.it = 0;
	} else {
		return(1);
	}

	// ........................................................................
	// Filling learning data

	create_learningdata(hmask, weights, himages, hld);


	// ........................................................................
	// Memory desallocation
	for (k=0 ; k<nbIm ; k++) {
		if (himages[k].dims) {
			free(himages[k].dims);
		}
	}
	if (hmask.dims) {
		free(hmask.dims);
	}
	if (hld.dims) {
		free(hld.dims);
	}

	return(err);
}


/******************************************************************************/
/* Create learning data from 3D images                                        */
/******************************************************************************/
int learningdata_3D(int nbIm, int pointsNb, HyperMat mask, double *weights,
					HyperMat images[NBIMAGES_MAX], double **learningData){

	int err = 0;
	const int nbDim = 3;
	const int nbComp = 1;
	int dimvar = 0;
	HyperMat hld;


	// ........................................................................
	// Memory allocation for learning data
	if (*learningData = (double*) calloc(pointsNb * nbIm * nbComp, sizeof(double))) {
		// Create an hypermatrix for learning data
		hld.dimsize = nbDim;
		dimvar = 0;
		if (hld.dims = (int *) calloc(hld.dimsize, sizeof(int))) {
			hld.dims[dimvar++] = pointsNb;
			hld.dims[dimvar++] = nbIm;
			hld.dims[dimvar++] = nbComp;
		} else {
			return 1;
		}
		hld.size = pointsNb * nbIm * nbComp;
		hld.R = (*learningData);
		hld.it = 0;
	} else {
		return(1);
	}

	// ........................................................................
	// Filling learning data

	create_learningdata(mask, weights, images, hld);


	// ........................................................................
	// Memory desallocation
	if (hld.dims) {
		free(hld.dims);
	}

	return(err);
}


/******************************************************************************/
/* Create learning data from 4D images                                        */
/******************************************************************************/
int learningdata_4D(int nbIm, int pointsNb, HyperMat mask, double *weights,
					HyperMat images[NBIMAGES_MAX], HyperMat *learningData) {

	int i;

	const int nbDim = 3;
	const int nbComp = images[0].dims[3];

	// ........................................................................
	// Memory allocation for learning data
	learningData->size = pointsNb * nbIm * nbComp;
	learningData->dimsize = nbDim;
	if (learningData->dims = (int *) calloc(learningData->dimsize, sizeof(int))) {
		i=0;
		learningData->dims[i++] = pointsNb;
		learningData->dims[i++] = nbIm;
		learningData->dims[i++] = nbComp;
	} else {
		return 1;
	}
	if (!(learningData->R = (double *) calloc(learningData->size, sizeof(double)))) {
		return 1;
	}
	learningData->it = 0;

	// ........................................................................
	// Filling learning data

	create_learningdata(mask, weights, images, *learningData);


	return(0);
}


/******************************************************************************/
/* Create learning data                                                       */
/******************************************************************************/
void create_learningdata(HyperMat mask, double *weights, HyperMat images[NBIMAGES_MAX], HyperMat learningData) {

	int nbPoints = learningData.dims[0];
	int nbIm = learningData.dims[1];
	int nbComp = learningData.dims[2];
	int nbP = 0;
	int coord;
	int i_fun, i_comp;

	while (nbP < nbPoints) {

		// Random generation of coordonnees
		coord = (int)(mask.size * (float)rand() / RAND_MAX);

		// Verify that the point is valid : the point must belong to the mask
		if (mask.R[coord]) {
			for(i_fun = 0 ; i_fun < nbIm ; i_fun++) {
				for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
					learningData.R[nbP + i_fun*nbPoints + i_comp*nbPoints*nbIm]
						= weights[i_fun] * images[i_fun].R[coord + i_comp*mask.size];
				}
			}
			nbP++;
		}
	}

}







