/******************************************************************************/
/* Functions used to mask a grey level image from Scilab                      */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"

#include "int_immaskgreylevel.h"

/******************************************************************************/
/* Interface Scilab <-> maskgreylevelimage                                    */
/******************************************************************************/
/* Creation : 17/08/2001 (V. COUVERT)                                         */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_immaskgreylevel(char *fname)
{
	int l_image, m_image, n_image,
	    l_mask, m_mask, n_mask,
	    *l_masked_image, m_masked_image, n_masked_image;

	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=2, maxrhs=2;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Ouput arguments */

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "d", &m_mask, &n_mask, &l_mask);

	maskgreylevelimage(stk(l_image), stk(l_mask), &l_masked_image, m_image, n_image, &ierr);

	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(3,"d", &m_image, &n_image, &l_masked_image);

	if(l_masked_image) {
		free(l_masked_image);
	}

	LhsVar(1)=3;

	return(0);
}

/******************************************************************************/
/* Mask function                                                              */
/******************************************************************************/
/* Creation : 17/08/2001 (V. COUVERT)                                         */
/* Last modification : 18/04/2002  (V. COUVERT)                               */
/******************************************************************************/
int maskgreylevelimage(double *image, double *mask, double **masked_image,
					   int width, int height, int *ierr)
{
	int x, y;
	
	/* Memory allocation */
	if (!(*masked_image = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}

	for (y = 0 ; y < height ; y++)
	{
		for (x = 0 ; x < width ; x++)
		{
			if(mask[x+y*width]==1)
			{
				(*masked_image)[x+y*width]=image[x+y*width];
			}
			else
			{
				(*masked_image)[x+y*width]=0.0;
			}
		}
	}
	*ierr=0;
	return(0);
}

