/******************************************************************************/
/* Functions used to mask a RGB image from Scilab                             */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"

#include "int_immaskrgb.h"

/******************************************************************************/
/* Interface Scilab <-> immaskrgb                                             */
/******************************************************************************/
/* Creation : 21/08/2001 (V. COUVERT)                                         */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_immaskrgb(char *fname)
{
	int l_redimage, m_redimage, n_redimage,
	    l_greenimage, m_greenimage, n_greenimage,
	    l_blueimage, m_blueimage, n_blueimage,
	    l_mask, m_mask, n_mask,
	    m_red_masked_image, n_red_masked_image,
	    m_green_masked_image, n_green_masked_image,
	    m_blue_masked_image, n_blue_masked_image;
	double *l_red_masked_image, *l_green_masked_image, *l_blue_masked_image;

	int ierr=0;

	int minlhs=3, maxlhs=3, minrhs=4, maxrhs=4;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */
	
	GetRhsVar(1, "d", &m_redimage, &n_redimage, &l_redimage);
	GetRhsVar(2, "d", &m_greenimage, &n_greenimage, &l_greenimage);
	GetRhsVar(3, "d", &m_blueimage, &n_blueimage, &l_blueimage);
	GetRhsVar(4, "d", &m_mask, &n_mask, &l_mask);

	maskrgbimage(stk(l_redimage), stk(l_greenimage), stk(l_blueimage), stk(l_mask), &l_red_masked_image, &l_green_masked_image, &l_blue_masked_image, m_redimage, n_redimage, &ierr);
	
	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(5,"d", &m_redimage, &n_redimage, &l_red_masked_image);
	CreateVarFromPtr(6,"d", &m_redimage, &n_redimage, &l_green_masked_image);
	CreateVarFromPtr(7,"d", &m_redimage, &n_redimage, &l_blue_masked_image);
	
	if (l_red_masked_image) {
		free(l_red_masked_image);
	}
	if (l_green_masked_image) {
		free(l_green_masked_image);
	}
	if (l_blue_masked_image) {
		free(l_blue_masked_image);
	}
	
	LhsVar(1)=5;
	LhsVar(2)=6;
	LhsVar(3)=7;

	return(0);
}

/******************************************************************************/
/* Mask function                                                              */
/******************************************************************************/
/* Creation : 21/08/2001 (V. COUVERT)                                         */
/* Last modification : 26/02/2002 (V. COUVERT)                                */
/******************************************************************************/
int maskrgbimage(double *redimage, double *greenimage, double *blueimage, 
				 double *mask, double **red_masked_image, double **green_masked_image,
				 double **blue_masked_image, int width, int height, int *ierr)
{
	int x, y;

	/* Memory allocation */
	if (!(*red_masked_image = (double*)calloc(width*height, sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}
	if (!(*green_masked_image = (double*)calloc(width*height, sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}
	if (!(*blue_masked_image = (double*)calloc(width*height, sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}
	
	for (y = 0 ; y < height ; y++)
	{
		for (x = 0 ; x < width ; x++)
		{
			if(mask[x+y*width]==1)
			{
				(*red_masked_image)[x+y*width]=redimage[x+y*width];
				(*green_masked_image)[x+y*width]=greenimage[x+y*width];
				(*blue_masked_image)[x+y*width]=blueimage[x+y*width];
			}
			else
			{
				(*red_masked_image)[x+y*width]=0;
				(*green_masked_image)[x+y*width]=0;
				(*blue_masked_image)[x+y*width]=0;
			}
		}
	}
	*ierr=0;
	return(0);
}

