/******************************************************************************/
/* Functions used to search the biggest object in an image from Scilab        */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"

#include "int_immaxconnex.h"

/******************************************************************************/
/* Interface Scilab <-> immaxconnex                                           */
/******************************************************************************/
/* Creation : 15/05/2002 (V. COUVERT)                                         */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_immaxconnex(char *fname)
{
	int l_image, m_image, n_image,
	    l_max, m_max, n_max,
	    m_object_image, n_object_image;
	double *l_object_image;

	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=2, maxrhs=2;

	CheckRhs(minrhs, maxrhs); // Input arguments
	CheckLhs(minlhs, maxlhs); // Ouput arguments 

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "d", &m_max, &n_max, &l_max);

	maxcompoconnex(stk(l_image), m_image, n_image, *stk(l_max), &l_object_image, &ierr);

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(3,"d", &m_image, &n_image, &l_object_image);

	if (l_object_image) {
		free(l_object_image);
	}

	LhsVar(1)=3;

	return(0);
}

/******************************************************************************/
/* Searching main function                                                    */
/******************************************************************************/
/* Creation : 15/05/2002 (V. COUVERT)                                         */
/* Last modification : 15/05/2002 (V. COUVERT)                                */
/******************************************************************************/
int maxcompoconnex(double *image, int width, int height, 
				   double max, double **object_image, int *ierr)
{
	int x=0, y=0, i=0;
	int *nb_occur, biggest, val;
	/* Memory allocation */
	if (!(*object_image = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(nb_occur = (int*)calloc(1, sizeof(int)*((int)max+1))))
	{
		*ierr=1;
		return(-1);
	}

	for(y=0;y<height;y++)
	{
		for(x=0;x<width;x++)
		{
			(*(nb_occur + ((int)(*(image+x+y*width)))))++;
		}
	}

	/* Searching for max value in nb_occur */
	biggest=1;
	for(i=1;i<=max;i++)
	{
		if( (*(nb_occur+i))>(*(nb_occur+biggest)))
			biggest=i;
	}

	for(y=0;y<height;y++)
	{
		for(x=0;x<width;x++)
		{
			if((int)(*(image+x+y*width))==biggest)
				(*object_image)[x+y*width]=1;
			else
				(*object_image)[x+y*width]=0;
		}
	}

	free(nb_occur);

	*ierr=0;
	return(0);
}

