/******************************************************************************/
/* Functions used to calculate an median filtered image from Scilab           */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"

#include "int_immedian.h"

/******************************************************************************/
/* Long comparison function                                                   */
/******************************************************************************/
/* Creation : 25/02/2002 (V. COUVERT)                                         */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int compare(double *doub1, double *doub2)
{
	if(*doub1>*doub2)
		return(-1);
	else if(*doub1==*doub2)
		return(0);
	else
		return(1);
}

/******************************************************************************/
/* Interface Scilab <-> immedian                                              */
/******************************************************************************/
/* Creation : 13/05/2002 (V. COUVERT)                                         */
/* Last modification : 33/05/2002 (V. COUVERT)                                */
/******************************************************************************/
int int_immedian(char *fname)
{
	int l_image, m_image, n_image,
	    l_kernel, m_kernel, n_kernel,
	    m_median_image, n_median_image,
	    l_mask, m_mask, n_mask;
	double *l_median_image;

	int ierr=0;

	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs, maxrhs); // Input arguments
	CheckLhs(minlhs, maxlhs); // Output arguments 
	
	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "d", &m_kernel, &n_kernel, &l_kernel);
	GetRhsVar(3, "d", &m_mask, &n_mask, &l_mask);

	median_image(stk(l_image), m_image, n_image, stk(l_kernel), m_kernel, n_kernel,
 &l_median_image, stk(l_mask), &ierr);

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(4,"d", &m_image, &n_image, &l_median_image);

	if (l_median_image) {
		free(l_median_image);
	}

	LhsVar(1)=4;

	return(0);
}

/******************************************************************************/
/* Median image computation function                                          */
/******************************************************************************/
/* Creation : 13/05/2001 (V. COUVERT)                                         */
/* Last modification : 13/05/2002 (V. COUVERT)                                */
/******************************************************************************/
int median_image(double *image, int width, int height, double *kernel,
				 int kernel_width, int kernel_height, double **median_image,
				 double *mask, int *ierr)
{
	static int x, y, xscrut, yscrut, nb_val=0;
	double *values;

	/* Memory allocation */
	if (!(*median_image = (double*)calloc(1, sizeof(double)*width*height)))
	{
		*ierr=1;
		return(-1);
	}
	if (!(values = (double*)calloc(1, sizeof(double)*kernel_width*kernel_height)))
	{
		*ierr=1;
		return(-1);
	}

	/* Filling median image */
	for(y=0 ; y<height ; y++)
	{
		for(x=0 ; x<width ; x++)
		{
			/* If the pixel is not masked */
			if(*(mask+x+y*width)!=0)
			{
				/* Searching for all values to consider */
				for(yscrut=-kernel_height/2 ; yscrut<=kernel_height/2 ; yscrut++)
				{
					for(xscrut=-kernel_width/2 ; xscrut<=kernel_width/2 ; xscrut++)
					{
						/* If value has to be count : kernel corresponding value is != 0*/
						if(kernel[xscrut+kernel_width/2+(yscrut+kernel_height/2)*kernel_width]!=0)
						{
							/* If we point a pixel in the image (to take border in account) and the pixel is not
 masked*/
							if( (x+xscrut>=0) && (x+xscrut<width) && (y+yscrut>=0) && (y+yscrut<height) &&
 ((*(mask+x+xscrut+(y+yscrut)*width))!=0) )
							{
								*(values+nb_val)=*(image+x+y*width+xscrut+yscrut*width);
								nb_val++;
							}
						}
					}
				}
				if(nb_val!=0)
				{
					qsort(values,nb_val,sizeof(double),compare);
					if((nb_val%2)==1)
					{
						(*median_image)[x+y*width]=*(values+nb_val/2);
					}
					else
					{
						(*median_image)[x+y*width]=(*(values+nb_val/2)+*(values+nb_val/2+1))/2;
					}
				}
				else
				{
					(*median_image)[x+y*width]=0;
				}
				nb_val=0;
			}
			else
			{
				(*median_image)[x+y*width]=*(image+x+y*width);
			}
		}
	}

	free(values);

	*ierr=0;
	return(0);
}

