/******************************************************************************/
/* Functions used to compute a threshold of an image from Scilab              */
/* Using OTSU method                                                          */
/******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"

#include "int_imotsu.h"

/******************************************************************************/
/* Interface Scilab <-> imotsu                                                */
/******************************************************************************/
/* Creation : 02/04/2002 (V. COUVERT)                                         */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/

int int_imotsu(char *fname)
{
	int l_image, m_image, n_image,
	    l_discri, m_discri=1, n_discri=1,
	    l_mask, m_mask, n_mask,
	    m_thresh, n_thresh,
	    l_thresh_nb, m_thresh_nb, n_thresh_nb;
	double *l_thresh;

	int ierr=0;

	int minlhs=2, maxlhs=2, minrhs=3, maxrhs=3;

	CheckRhs(minrhs, maxrhs); // Input arguments 
	CheckLhs(minlhs, maxlhs); // Ouput arguments 

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "d", &m_mask, &n_mask, &l_mask);
	GetRhsVar(3, "i", &m_thresh_nb, &n_thresh_nb, &l_thresh_nb);

	CreateVar(4, "d", &m_discri, &n_discri, &l_discri);


	otsu(stk(l_image),stk(l_mask),m_image,n_image,&l_thresh,*istk(l_thresh_nb),stk(l_discri),&ierr);

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	n_thresh=*istk(l_thresh_nb);
	m_thresh=1;

	CreateVarFromPtr(5,"i", &n_thresh, &m_thresh, &l_thresh);

	if(l_thresh) {
		free(l_thresh);
	}

	LhsVar(1)=5;
	LhsVar(2)=4;
	return(0);
}

/******************************************************************************/
/* threshold computing                                                        */
/******************************************************************************/
/* Creation : 02/04/2002 (V. COUVERT)                                         */
/* Last modification : 19/04/2002 (V. COUVERT)                                */
/******************************************************************************/
int otsu(double *image, double *mask, int width, int height, int **thresh,
		 int thresh_nb, double *discriminant, int *ierr)
{
	int x, y, i, j,val, max=0, hist_size=65536, *histo, nb_pts=0, k1=0, k2=0, k=0;
	double *histo_prob, sigmaB2_max=0.0, sigmaB2=0.0, muT=0.0, omega0=0.0, omega1=0.0, omega2=0.0, mu0=0.0, mu1=0.0, mu2=0.0, sigmaT2;

     	/* Memory allocation */
    	if (!(histo = (int*)calloc(1, sizeof(int)*hist_size)))
    	{
		*ierr=1;
		return(-1);
    	}
	
	/* Histogram computing */
	for(y = 0 ; y < height ; y++)
	{
 	   	for(x = 0 ; x < width ; x++)
  	  	{
			if(*(mask+x+y*width)==1)
			{
				val=*(image+x+y*width);
				(*(histo+val))++;
				nb_pts++;
				if(*(image+x+y*width)>max)
				{
					max=*(image+x+y*width);
				}
			}
		}
   	}
 	if(max<=255)
   	{
   		hist_size=256;
   	}
   	if((max>255)&&(max<=4096))
   	{
   		hist_size=4096;
   	}
   	if((max>4095)&&(max<=65535))
   	{
   		hist_size=65536;
   	}
   	if (!((histo) = realloc(histo, sizeof(int)*hist_size)))
	{
		*ierr=1;
		return(-1);
	}
	/* End of histogram computation */

	/* Computing probability histogram */
	if (!(histo_prob = (double*)calloc(1, sizeof(double)*hist_size)))
	{
		*ierr=1;
		return(-1);
	}
	for(i=0;i<max;i++)
	{
		histo_prob[i]=histo[i];
		histo_prob[i]=histo_prob[i]/nb_pts;
	}
	/* End of probability histogram computation */
	if (!(*thresh = (int*)calloc(1, sizeof(int)*thresh_nb)))
 	{
		*ierr=1;
		return(-1);
 	}

	/* OTSU */
	if(thresh_nb==1)
	{
		/* muT computation */
		for(i=0;i<max;i++)
			muT+=(i+1)*histo_prob[i];
	
		for(i=0;i<max-1;i++)
		{
	
			for(j=0;j<=i;j++)
				omega0+=histo_prob[j];
			omega1=1.0-omega0;
			for(j=0;j<=i;j++)
				mu0+=(j+1)*histo_prob[j];
			mu0/=omega0;
			for(j=i+1;j<max;j++)
				mu1+=(j+1)*histo_prob[j];
			mu1/=omega1;
			sigmaB2=omega0*((mu0-muT)*(mu0-muT))+omega1*((mu1-muT)*(mu1-muT));
			if(sigmaB2>sigmaB2_max)
			{
				sigmaB2_max=sigmaB2;
				(*thresh)[0]=i;
			}
			/* ReInit */
			omega0=0.0;
			omega1=0.0;
			mu0=0.0;
			mu1=0.0;
		}
	}
	if(thresh_nb==2)
	{
		/* muT computation */
		for(i=0;i<max;i++)
			muT+=(i+1)*histo_prob[i];
	
		for(k1=0;k1<max-1;k1++)
		{
			for(j=0;j<=k1;j++)
				omega0+=histo_prob[j];
			
			for(j=0;j<=k1;j++)
				mu0+=(j+1)*histo_prob[j];
			mu0/=omega0;

			for(k2=k1+1; k2<max-1;k2++)
			{
				for(j=k1+1;j<=k2;j++)
					omega1+=histo_prob[j];
				omega2=1.0-omega0-omega1;
				
				for(j=k1+1;j<=k2;j++)
					mu1+=(j+1)*histo_prob[j];
				mu1/=omega1;
				
				for(j=k2+1;j<max;j++)
					mu2+=(j+1)*histo_prob[j];
				mu2/=omega2;

			
	sigmaB2=omega0*((mu0-muT)*(mu0-muT))+omega1*((mu1-muT)*(mu1-muT))+omega2*((mu2-muT)*
(mu2-muT));

				if(sigmaB2>sigmaB2_max)
				{
					sigmaB2_max=sigmaB2;
					(*thresh)[0]=k1;
					(*thresh)[1]=k2;
				}
				omega1=0.0;
				omega2=0.0;
				mu1=0.0;
				mu2=0.0;
				
			}
			/* ReInit */
			omega0=0.0;
			mu0=0.0;
		}
	}

	/* discriminant computation */
	for(i=0;i<max;i++)
		sigmaT2+=((i+1)-muT)*((i+1)-muT)*histo_prob[i];
	*discriminant=sigmaB2_max/sigmaT2;

	free(histo);
	free(histo_prob);

	*ierr=0;
	return(0);
}
