/******************************************************************************/
/* Functions used to compute a grey level profile                             */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include "stack-c.h"

#include "int_improfilegreylevel.h"

/******************************************************************************/
/* Interface Scilab <-> improfilegreylevel                                    */
/******************************************************************************/
/* Creation : 17/08/2001 (V. COUVERT)                                         */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_improfilegreylevel(char *fname)
{
	int l_image, m_image, n_image,
	    l_xsel, m_xsel, n_xsel,
	    l_ysel, m_ysel, n_ysel,
	    m_profile, n_profile=1;
	double *l_profile;

	int ierr=0;
	    
	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	CheckRhs(minrhs, maxrhs); // Input arguments 
	CheckLhs(minlhs, maxlhs); // Output arguments 

	GetRhsVar(1, "d", &m_image, &n_image, &l_image);
	GetRhsVar(2, "i", &m_xsel, &n_xsel, &l_xsel);
	GetRhsVar(3, "i", &m_ysel, &n_ysel, &l_ysel);

	greylevelprofile(stk(l_image), m_image, n_image, &l_profile, &m_profile, istk(l_xsel), istk(l_ysel), &ierr);

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}
	
	CreateVarFromPtr(4,"d", &m_profile, &n_profile, &l_profile);

	if (l_profile) {
		free(l_profile);
	}
	
	LhsVar(1)=4;
	
	return(0);
}


/******************************************************************************/
/* Computing function                                                         */
/******************************************************************************/
/* Creation : 17/08/2001 (V. COUVERT)                                         */
/* Last modification : 14/06/2005 (M. TALLUR)                                 */
/******************************************************************************/
int greylevelprofile(double *image, int width, int height, double **profile,
					 int *profile_size, int *xsel, int *ysel, int *ierr)
{
	int i, deltax=0, deltay=0 ;
	double length, angle, cos_angle, sin_angle, x, y;

	deltax = xsel[1]-xsel[0];
	deltay = ysel[1]-ysel[0];
	
	length = deltax*deltax + deltay*deltay;
	length = sqrt(length);
	
	angle = atan2(deltax, deltay);
	cos_angle = cos(angle); 
	sin_angle = sin(angle);
	
	// Memory allocation
	if (!(*profile = (double*)calloc(ceil(length), sizeof(double))))
	{ 
		*ierr=1;
		return(-1);
	}
	
	for (i = 0; i<floor(length); i++)
	{
	  x = (xsel[0]-1) + i * sin_angle;
	  y = (ysel[0]-1) + i * cos_angle;
		//printf("[%d:%lf,%lf] ",i,x,y);
		(*profile)[i] = image[(long)(floor(x)+floor(y)*width)];
	}
	
	*profile_size=floor(length);

	*ierr=0;
	return(0);
}


