/******************************************************************************/
/* Functions used to compute a color profile                                  */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include "stack-c.h"
#include "int_improfilergb.h"

/******************************************************************************/
/* Interface Scilab <-> improfilergb                                          */
/******************************************************************************/
/* Creation : 21/08/2001 (V. COUVERT)                                         */
/* Last modification : 29/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_improfilergb(char *fname)
{
	static int l_redimage, m_redimage, n_redimage,
	    l_greenimage, m_greenimage, n_greenimage,
	    l_blueimage, m_blueimage, n_blueimage,
	    l_xsel, m_xsel, n_xsel,
	    l_ysel, m_ysel, n_ysel,
	    m_redprofile, n_redprofile=1,
	    m_greenprofile, n_greenprofile=1,
	    m_blueprofile, n_blueprofile=1;
	double *l_redprofile, *l_greenprofile, *l_blueprofile;

	int ierr=0;
	    
	int minlhs=3, maxlhs=3, minrhs=5, maxrhs=5;
	
	CheckRhs(minrhs, maxrhs); // Input arguments 
	CheckLhs(minlhs, maxlhs); // Output arguments 

	
	GetRhsVar(1, "d", &m_redimage, &n_redimage, &l_redimage);
	GetRhsVar(2, "d", &m_greenimage, &n_greenimage, &l_greenimage);
	GetRhsVar(3, "d", &m_blueimage, &n_blueimage, &l_blueimage);
	GetRhsVar(4, "i", &m_xsel, &n_xsel, &l_xsel);
	GetRhsVar(5, "i", &m_ysel, &n_ysel, &l_ysel);

	rgbprofile(stk(l_redimage), stk(l_greenimage), stk(l_blueimage), m_redimage, n_redimage, &l_redprofile, &l_greenprofile, &l_blueprofile, &m_redprofile, istk(l_xsel), istk(l_ysel), &ierr);

	// In case of error 
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVarFromPtr(6,"d", &m_redprofile, &n_redprofile, &l_redprofile);
	CreateVarFromPtr(7,"d", &m_redprofile, &n_greenprofile, &l_greenprofile);
	CreateVarFromPtr(8,"d", &m_redprofile, &n_blueprofile, &l_blueprofile);
	
	if (l_redprofile) {
		free(l_redprofile);
	}
	if (l_greenprofile) {
		free(l_greenprofile);
	}
	if (l_blueprofile) {
		free(l_blueprofile);
	}
	
	LhsVar(1)=6;
	LhsVar(2)=7;
	LhsVar(3)=8;

	return(0);
}


/******************************************************************************/
/* Computing function                                                         */
/******************************************************************************/
/* Creation : 21/08/2001 (V. COUVERT)                                         */
/* Last modification : 14/06/2005 (M. TALLUR)                                 */
/******************************************************************************/
int rgbprofile(double *redimage, double *greenimage, double *blueimage, 
			   int width, int height, 
			   double **redprofile, double **greenprofile, double **blueprofile, 
			   int *profile_size, int *xsel, int *ysel, int *ierr)
{

	int nb_pts=0, deltax=0, deltay=0, i ;
	double length, angle, cos_angle, sin_angle, x, y;

	deltax = xsel[1]-xsel[0];
	deltay = ysel[1]-ysel[0];
	
	length = deltax*deltax + deltay*deltay;
	length = sqrt(length);
	
	angle = atan2(deltax, deltay);
	cos_angle = cos(angle); 
	sin_angle = sin(angle);
	
	// Memory allocation 
	if (!(*redprofile = (double*)calloc(ceil(length), sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}
	if (!(*greenprofile = (double*)calloc(ceil(length), sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}
	if (!(*blueprofile = (double*)calloc(ceil(length), sizeof(double))))
	{
		*ierr=1;
		return(-1);
	}


	for (i = 0; i<floor(length); i++)
	{
		x = (xsel[0]-1) + i * sin_angle;
		y = (ysel[0]-1) + i * cos_angle;
		
		(*redprofile)[i] = redimage[(long)(floor(x)+floor(y)*width)];
		(*greenprofile)[i] = greenimage[(long)(floor(x)+floor(y)*width)];
		(*blueprofile)[i] = blueimage[(long)(floor(x)+floor(y)*width)];
	}

		
	*profile_size=floor(length);
	
	*ierr=0;
	return(0);
}

