/******************************************************************************/
/* Fonctions used to open a image file from Scilab                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <fcntl.h>
#include "stack-c.h"

#include "int_imread.h"

/******************************************************************************/
/* Interface Scilab<->read_image                                              */
/******************************************************************************/
/* Creation : 09/07/2001 (V. COUVERT)                                         */
/* Last modification : 17/08/2005 (M. TALLUR)                                 */
/******************************************************************************/
int int_imread(char *fname)
{
	unsigned int l_filename, m_filename, n_filename,
		*l_redimage, *l_greenimage, *l_blueimage,
		l_colour, m_colour=1, n_colour=1, colour,
		m_image, n_image;

	int minlhs=2, maxlhs=4, minrhs=1, maxrhs=1;

	int ierr=0;

	CheckRhs(minrhs, maxrhs); // Input arguments 
	CheckLhs(minlhs, maxlhs); // Output arguments

	GetRhsVar(1, "c", &m_filename, &n_filename, &l_filename);

	CreateVar(2, "i", &m_colour, &n_colour, &l_colour);

	read_image(cstk(l_filename), &l_redimage, &l_greenimage, &l_blueimage, &m_image, &n_image, &colour, &ierr);

	// In case of error
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	*istk(l_colour) = colour;
	LhsVar(1)=2;

	if (colour)
	{
		CreateVarFromPtr(3, "i", &m_image, &n_image, &l_redimage);
		CreateVarFromPtr(4, "i", &m_image, &n_image, &l_greenimage);
		CreateVarFromPtr(5, "i", &m_image, &n_image, &l_blueimage);
		LhsVar(2)=3;
		LhsVar(3)=4;
		LhsVar(4)=5;
	}
	else
	{
		CreateVarFromPtr(3, "i", &m_image, &n_image, &l_redimage);
		// we also provide some useless stuff so that scilab doesn't complain
		CreateVar(4, "i", &m_colour, &n_colour, &l_colour);
		CreateVar(5, "i", &m_colour, &n_colour, &l_colour);
		LhsVar(2)=3;
		LhsVar(3)=4;
		LhsVar(4)=5;
	}

	if(l_redimage) free(l_redimage);
	if(l_greenimage) free(l_greenimage);
	if(l_blueimage) free(l_blueimage);

	return(0);
}


/******************************************************************************/
/* Function which call the function needed according to the file type         */
/******************************************************************************/
/* Creation : 09/07/2001 (V. COUVERT)                                         */
/* Last modification : 25/08/2005 (M. TALLUR)                                 */
/******************************************************************************/
int read_image(char *filename,
			   unsigned int **redimage, unsigned int **greenimage,
			   unsigned int **blueimage, int *m, int *n, int *colour, int *ierr)
{
	char extension[5];
	int i, j;
	int lg = 0;
	int point = 0;

	*redimage = NULL; *greenimage = NULL; *blueimage = NULL;
	*m = 0; *n = 0;


	// Reading of the file extension
/*	for(i=0;filename[i]!='.';i++);
	i++;
	for(j=0;filename[i]!='\0';i++,j++)
	{
		extension[j]=filename[i];
	}
	extension[j]='\0';
	*/

	// Virginie
	for(i=0 ; filename[i]!='\0' ; i++){
		lg++;
	}
	for(i=lg-1 ; i>=0 && point==0; i--) {
		if (filename[i]=='.') {
			point = i;
		}
	}
	for(i=point+1, j=0 ; i<=lg && j<5 ;i++,j++) {
		extension[j] = filename[i];
	}


	// Call of the function associated to the file format
	if((strcmp(extension,"pgm")==0))
	{
		*ierr=read_pgm(filename, redimage, m, n);
		*colour = 0;
		return(0);
	}
	else if((strcmp(extension,"ppm")==0))
	{
		*ierr=read_ppm(filename, redimage, greenimage, blueimage, m, n);
		*colour = 1;
		return(0);
	}

	else if((strcmp(extension,"tif")==0))
	{
		*ierr=read_tif(filename, redimage, m, n);
		*ierr=0;
		*colour = 0;
		return(0);
	}

	else if((strcmp(extension,"dcm")==0))
	{
		*ierr=read_dcm(filename, redimage, m, n);
		*colour = 0;
		return(0);
	}

	else if((strcmp(extension,"png")==0))
	{
		*ierr=read_png(filename, redimage, greenimage, blueimage, m, n, colour);
		return(0);
	}

	else if((strcmp(extension,"jpg")==0))
	{
		*ierr=read_jpg(filename, redimage, greenimage, blueimage, m, n, colour);
		return(0);
	}

	else if((strcmp(extension,"bmp")==0))
	{
		*ierr=read_bmp(filename, redimage, greenimage, blueimage, m, n, colour);
		return(0);
	}

	else
	{
		// Other cases can't exist because of "imread()" Scilab function
		*ierr=1;
		return(-1);
	}
}

/******************************************************************************/
/* Function used to read a PGM extension file                                 */
/* 8-bit grey level image                                                     */
/******************************************************************************/
/* Creation : 09/07/2001 (V. COUVERT)                                         */
/* Last modification : 12/02/2002 (V. COUVERT)                                */
/******************************************************************************/
int read_pgm(char *filename, unsigned int **image, int *m, int *n)
{
	int id; // File identificator
	int image_width=0, image_height=0, nb_colors, x, y, i=0;
	char magic[3], c, buf[80];
	unsigned char *data;

    // File opening : read only/text mode
	id=open(filename, O_RDONLY);// | O_TEXT);

	if(id==-1) // If file doesn't exist or read access denied
	{
		sciprint("File %s doesn't exist or read access denied.\n", filename);
		if (!(*image = (unsigned int*)calloc(1, 1)))
		{
			return(1);
		}
		*m=1;
		*n=1;
		image[0][0]=-1;
		return(0);
	}
	else
	{
		// File header reading
		read(id, magic, 3); // Magic number
		magic[2]='\0';
		// Comments reading if there are!
		read(id, &c, 1);
		if(c=='\n')
			read(id, &c, 1);
		while(c=='#')
		{
			while( (read(id, &c, 1)==1) && (c!='\n') );
			read(id, &c, 1);
		}
		/* Width and height reading */
		buf[0]=c;
		i=1;
		while( (read(id, &buf[i], 1)==1) && (buf[i]!='\n') && (buf[i]!='\r') && (i<79))
		{
			i++;
		}
		buf[i]='\0';
		sscanf(buf, "%d %d", &image_width, &image_height);
		/* Comments reading if there are! */
		read(id, &c, 1);
		while(c=='#')
		{
			while( (read(id, &c, 1)==1) && (c!='\n') );
			read(id, &c, 1);
		}
		/* Colors reading */
		buf[0]=c;
		i=1;
		while( (read(id, &buf[i], 1)==1) && (buf[i]!='\n') && (buf[i]!='\n') && (i<79))
		{
			i++;
		}
		buf[i]='\0';
		sscanf(buf, "%d", &nb_colors);

		/* Memory allocation */
		if (!(data = (unsigned char*)calloc(image_width*image_height, sizeof(unsigned char))))
		{
			return(1);
		}
		if (!(*image = (unsigned int*)calloc(image_width*image_height, sizeof(unsigned int))))
		{
			return(1);
		}
		*m=image_width;
		*n=image_height;

		/* Data reading */
		read(id, data, image_width*image_height);

		close (id);

    	/* Data conversion from 'int' to 'char' */
    	for(y=0 ;y<image_height ; y++)
    	{
      		for(x=0 ; x<image_width ; x++)
      		{
      			(*image)[x+y*image_width]=*(data+x+y*image_width);
      		}
      	}

		free(data);
 	}
	return(0);
}


/******************************************************************************/
/* Function used to read a PPM extension file                                 */
/* 24-bit color image                                                         */
/******************************************************************************/
/* Creation : 09/07/2001 (V. COUVERT)                                         */
/* Last modification : 12/02/2002 (V. COUVERT)                                */
/******************************************************************************/
int read_ppm(char *filename, unsigned int **redimage, unsigned int **greenimage, unsigned
 int **blueimage, int *m, int *n)
{
	int id; /* File identificator */
	int image_width=0, image_height=0, nb_colors, x, y, i=0;
	char magic[3], c, buf[80];
	unsigned char *data;

    /* File opening : read only/text mode */
	id=open(filename,O_RDONLY);// | O_TEXT);

	if(id==-1) /* If file doesn't exist or read access denied */
	{
		sciprint("File %s doesn't exist or read access denied.\n", filename);
		if (!(*redimage = (unsigned int*)calloc(1, 1)))
		{
			return(1);
		}
		if (!(*greenimage = (unsigned int*)calloc(1, 1)))
		{
			return(1);
		}
		if (!(*blueimage = (unsigned int*)calloc(1, 1)))
		{
			return(1);
		}
		*m=1;
		*n=1;
		redimage[0][0]=-1;
		greenimage[0][0]=-1;
		blueimage[0][0]=-1;
		return(0);
	}
	else
	{
		/* File header reading */
		read(id, magic, 3); /* Magic number */
		magic[2]='\0';
		/* Comments reading if there are! */
		read(id, &c, 1);
		if(c=='\n')
			read(id, &c, 1);
		while(c=='#')
		{
			while( (read(id, &c, 1)==1) && (c!='\n'));
			read(id, &c, 1);
		}
		/* Width and height reading */
		buf[0]=c;
		i=1;
		while( (read(id, &buf[i], 1)==1) && (buf[i]!='\n') && (buf[i]!='\r') && (i<79))
		{
			i++;
		}
		buf[i]='\0';
		sscanf(buf, "%d %d", &image_width, &image_height);
		/* Comments reading if there are! */
		read(id, &c, 1);
		while(c=='#')
		{
			while( (read(id, &c, 1)==1) && (c!='\n') );
			read(id, &c, 1);
		}
		/* Colors reading */
		buf[0]=c;
		i=1;
		while( (read(id, &buf[i], 1)==1) && (buf[i]!='\n') && (buf[i]!='\n') && (i<79))
		{
			i++;
		}
		buf[i]='\0';
		sscanf(buf, "%d", &nb_colors);

    	/* Memory allocation */
    	if (!(data = (unsigned char*)calloc(3*image_width*image_height, sizeof(unsigned
 char))))
    	{
			return(1);
    	}
    	if (!(*redimage = (unsigned int*)calloc(image_width*image_height, sizeof(unsigned
 int))))
    	{
			return(1);
    	}
    	if (!(*greenimage = (unsigned int*)calloc(image_width*image_height, sizeof(unsigned
 int))))
    	{
			return(1);
    	}
    	if (!(*blueimage = (unsigned int*)calloc(image_width*image_height, sizeof(unsigned
 int))))
    	{
			return(1);
    	}
    	*m=image_width;
    	*n=image_height;

    	/* Data reading */
		read(id, data, 3*image_width*image_height);

		close (id);

    	/* Data conversion from 'int' to 'char' */
		/* Data is converted into 3 matrices R, V, B */
    	for(y=0 ;y<image_height ; y++)
    	{
      		for(x=0 ; x<image_width ; x++)
      		{
      			(*redimage)[x+y*image_width]=*(data+3*(x+y*image_width));
      			(*greenimage)[x+y*image_width]=*(data+3*(x+y*image_width)+1);
      			(*blueimage)[x+y*image_width]=*(data+3*(x+y*image_width)+2);
      		}
      	}

		free(data);
	}	return(0);
}

/******************************************************************************/
/* Function used to read a TIFF extension file                                */
/* 16-bit grey level image                                                    */
/* Be careful that a TIFF file can contain others image formats               */
/******************************************************************************/
/* Creation : 15/03/2002 (V. COUVERT)                                         */
/* Last modification : 15/03/2002 (V. COUVERT)                                */
/******************************************************************************/
#include "tiffio.h"
int read_tif(char *filename, unsigned int **image, int *m, int *n)
{
	TIFF* tif;
	uint32 ImageWidth, ImageHeight;
	uint16 BitsPerSample, Compression, PhotometricInterpretation;

	tdata_t buf;
	tstrip_t strip;

	int i=0, j=0;
	unsigned char *tmp;

	// File opening : read only mode 
	tif = TIFFOpen(filename, "r");
	if(tif==0)
	{
		sciprint("File %s doesn't exist or read access denied.\n", filename);
		if (!(*image = (unsigned int*)calloc(1, 1)))
		{
			return(1);
		}
		*m=1;
		*n=1;
		image[0][0]=-1;
		return(0);
	}
	else
	{
		TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &ImageWidth);
		TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &ImageHeight);
		TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &BitsPerSample);
		TIFFGetField(tif, TIFFTAG_COMPRESSION, &Compression);
		TIFFGetField(tif, TIFFTAG_PHOTOMETRIC, &PhotometricInterpretation);

		if((BitsPerSample!=(uint16) 16) || (Compression!=(uint16) 1) ||
 (PhotometricInterpretation!=(uint16) 1))
		{
			sciprint("File is not a 16-bits greyscale TIFF file.\n");
			if (!(*image = (unsigned int*)calloc(1, 1)))
			{
				return(1);
			}
			*m=1;
			*n=1;
			image[0][0]=-1;
			TIFFClose(tif);
			return(0);
		}

		// Data reading 
		*m=(int) ImageWidth;
		*n=(int) ImageHeight;

		if (!(*image = (unsigned int*)calloc((*m)*(*n), sizeof(unsigned int))))
		{
			TIFFClose(tif);
			return(1);
		}

		buf = _TIFFmalloc(TIFFStripSize(tif));
		for (strip = 0; strip < TIFFNumberOfStrips(tif); strip++)
		{
			TIFFReadEncodedStrip(tif, strip, buf, (tsize_t) -1);

			tmp=(char*) buf;

			// Data conversion from 'char' to 'int' 
			for(i=0;i<TIFFStripSize(tif);i+=2)
			{
				(*image)[i/2+j]=*(tmp+i+1);
				(*image)[i/2+j]=((*image)[i/2+j])*256 + *(tmp+i);

				/*(*image)[0]=*m;
				(*image)[1]=*n;
				(*image)[2]=i/2+j;
				(*image)[3]=TIFFNumberOfStrips(tif);*/
			}
			j+=i/2;
		}

		_TIFFfree(buf);

		TIFFClose(tif);
	}
	return(0);
}

/******************************************************************************/
/* Function used to read a DCM extension file                                 */
/* 12-bit grey level image                                                    */
/******************************************************************************/
/* Creation : 20/03/2002 (V. COUVERT)                                         */
/* Last modification : 20/03/2002 (V. COUVERT)                                */
/******************************************************************************/
#include "dicom.h"
#include "condition.h"
#include "lst.h"
#include "dicom_objects.h"
#include "dicom_uids.h"
#include "ctnthread.h"

int read_dcm(char *filename, unsigned int **image, int *m, int *n)
{
	DCM_OBJECT *dcmObject = NULL;
	DCM_ELEMENT dcmElement;
	CONDITION cond;
	unsigned long nOptions = DCM_ORDERLITTLEENDIAN | DCM_PART10FILE;
	int width, height, i=0;
	void *dummPointer;
	U32 rtnSize;
	void *ctx=NULL, *ctx1=NULL, *ctx2=NULL;
	short *image_data;

	/* Openning the file */
	cond = DCM_OpenFile (filename, nOptions, &dcmObject);

	if(cond!=DCM_NORMAL) /* Opening file failed : maybe it is not a PART10 file... */
	{
		nOptions &= ~(DCM_PART10FILE);
		cond = DCM_OpenFile (filename, nOptions, &dcmObject);
		if(cond!=DCM_NORMAL) /* Opening file has failed : try with BIGENDIANORDER */
		{
			nOptions |= DCM_PART10FILE;
			nOptions &= ~(DCM_ORDERMASK);
			nOptions |= DCM_ORDERBIGENDIAN;
			cond = DCM_OpenFile (filename, nOptions, &dcmObject);
			if(cond!=DCM_NORMAL) /* Opening file failed : maybe it is not a PART10 file... */
			{
				nOptions &= ~(DCM_PART10FILE);
				cond = DCM_OpenFile (filename, nOptions, &dcmObject);
				if(cond!=DCM_NORMAL)
				{
					sciprint("File %s doesn't exist or read access denied.\n", filename);
					if (!(*image = (unsigned int*)calloc(1, 1)))
					{
						return(1);
					}
					*m=1;
					*n=1;
					image[0][0]=-1;
					return(0);
				}
			}
		}
	}

	/* Reading image width */
	dcmElement.tag=DCM_IMGCOLUMNS;
	DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
	dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
	DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx);
	width=(int)(*(U16 *)dcmElement.d.ot);

	/* Reading image height */
	dcmElement.tag=DCM_IMGROWS;
	DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
	dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
	DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx1);
	height=(int)(*(U16 *)dcmElement.d.ot);

	*m=width;
	*n=height;

	/* Reading image data */
	dcmElement.tag=DCM_PXLPIXELDATA;
	DCM_GetElement(&dcmObject, dcmElement.tag, &dcmElement);
	dcmElement.d.ot=(void *) realloc ((void *)dcmElement.d.ot, dcmElement.length);
	DCM_GetElementValue(&dcmObject, &dcmElement, &rtnSize, &ctx2);
	image_data=(short *)(dcmElement.d.ot);

	/* Memory allocation */
	if (!(*image = (unsigned int*)calloc((*m)*(*n), sizeof(unsigned int))))
	{
		return(1);
	}

	/* Passing image_data as int */
	for(i=0;i<width*height;i++)
		(*image)[i]=*(image_data+i);

	free(image_data);

	/* Closing file */
	DCM_CloseObject(&dcmObject);
	return(0);

}

/******************************************************************************/
/* Function used to read a PNG extension file                                 */
/* 1-bit grey level image to 48-bit color image, lossless compression         */
/******************************************************************************/
/* Creation : 26/08/2005 (M. TALLUR)                                          */
/******************************************************************************/

#include "png.h"
int read_png(char *filename, unsigned int **redimage, unsigned int **greenimage, unsigned int **blueimage, int *m, int *n, int *colour)
{
	FILE *fp ;
	png_structp png_ptr = NULL;
	png_infop info_ptr = NULL;
	char header[8];
	int i, u;

	fp = fopen(filename, "rb");
	if (!fp)
	{
		// cannot open file
        	return (1);
	}

	fread(header, 1, 8, fp);

	if (png_sig_cmp((png_bytep)&header, 0, 8))
	{
		// supplied file is not a png picture file
		sciprint("ImRead : supplied .png file not recognized by libpng");
		return (1);
	}

	if ((png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, (png_voidp)NULL, NULL, NULL)))
		if ((info_ptr = png_create_info_struct(png_ptr)))
		{
			png_bytep *row_pointers = NULL;
			int width, height, bit_depth, color_type, interlace_type, compression_type, filter_method;

			png_init_io(png_ptr, fp);

			png_set_sig_bytes(png_ptr, 8);

			png_read_png(png_ptr, info_ptr, PNG_TRANSFORM_STRIP_ALPHA|PNG_TRANSFORM_PACKING|PNG_TRANSFORM_EXPAND, NULL);

			row_pointers = png_get_rows(png_ptr, info_ptr);

			png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_type, &compression_type, &filter_method);

			if (!(*redimage = (unsigned int *)malloc(sizeof(unsigned int)*width*height)))
				return (1);

			*colour = ((color_type == PNG_COLOR_TYPE_RGB) || (color_type == PNG_COLOR_TYPE_RGB_ALPHA)) ? 1 : 0;


			if (*colour)
			{
				if (!(*greenimage = (unsigned int *)malloc(sizeof(unsigned int)*width*height)))
					return 1;
				if (!(*blueimage = (unsigned int *)malloc(sizeof(unsigned int)*width*height)))
					return 1;

				if (bit_depth == 16)
				{
					for (i=0; i<height; i++)
						for (u=0; u<width; u++)
						{
							(*redimage)[u + i*width] = ((row_pointers[i][2*3*u]) << 8) | row_pointers[i][2*3*u+1];
							(*greenimage)[u + i*width] =  ((row_pointers[i][2*(3*u+1)]) << 8) | row_pointers[i][2*(3*u+1)+1];
							(*blueimage)[u + i*width] = ((row_pointers[i][2*(3*u+2)]) << 8) | row_pointers[i][2*(3*u+2)+1];
						}
				}
				else
				{
					for (i=0; i<height; i++)
						for (u=0; u<width; u++)
						{
							(*redimage)[u + i*width] = row_pointers[i][3*u];
							(*greenimage)[u + i*width] = row_pointers[i][(bit_depth/8)*(3*u+1)];
							(*blueimage)[u + i*width] = row_pointers[i][(bit_depth/8)*(3*u+2)];
						}
				}
			}
			else
			{
				if (bit_depth == 16)
				{
					for (i=0; i<height; i++)
						for (u=0; u<width; u++)
							(*redimage)[u + i*width] = ((row_pointers[i][2*u]) << 8) | row_pointers[i][(bit_depth/8)*u+1];
				}
				else
				{
					for (i=0; i<height; i++)
						for (u=0; u<width; u++)
							(*redimage)[u + i*width] = row_pointers[i][u];
				}
			}

			*m = width;
			*n = height;

			//printf("Image PNG '%s' : %dx%d en %d bits (col %d)\n", filename, width, height, bit_depth, *colour);
		}


	// Releasing memory

	if (info_ptr)
		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);

	if (png_ptr)
		png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);

	return (0);
}

/******************************************************************************/
/* Function used to read a JPG extension file                                 */
/******************************************************************************/
/* Creation : 26/08/2005 (M. TALLUR)                                          */
/******************************************************************************/

#include "jpeglib.h"
int read_jpg(char *filename, unsigned int **redimage, unsigned int **greenimage, unsigned int **blueimage, int *m, int *n, int *colour)
{
	int width, height, i, u;
        struct jpeg_decompress_struct cinfo;
        struct jpeg_error_mgr jerr;
	JSAMPROW row_pointer = NULL;
	FILE * infile;

	// Allocate and initialize a JPEG decompression object.
	cinfo.err = jpeg_std_error(&jerr);
        jpeg_create_decompress(&cinfo);

	// Specify the source of the compressed data (eg, a file).
        if ((infile = fopen(filename, "rb")) == NULL) {
		sciprint("ImRead : cannot open .jpg file\n");
		return(1);
        }
        jpeg_stdio_src(&cinfo, infile);

	// Call jpeg_read_header() to obtain image info.
	jpeg_read_header(&cinfo, TRUE);

	jpeg_start_decompress(&cinfo);

	/*
	output_width            image width and height, as scaled
        output_height
        out_color_components    # of color components in out_color_space
        output_components       # of color components returned per pixel
	*/

	//printf("Lecture '%s' %dx%d (%d composantes)\n", filename, cinfo.output_width, cinfo.output_height, cinfo.out_color_components);

	width = cinfo.output_width;
	height = cinfo.output_height;
	if (!(row_pointer = (JSAMPROW)malloc(width*cinfo.out_color_components*sizeof(JSAMPLE))))
		return (1);

	if (!(*redimage = (unsigned int *)malloc(sizeof(unsigned int)*width*height)))
		return (1);

	switch(cinfo.out_color_components)
	{
		case 1: // Grayscales picture
			*colour = 0;
			
			for (u=0; u<height; u++)
			{
				jpeg_read_scanlines(&cinfo, &row_pointer, 1);
				for (i=0; i<width; i++)
					(*redimage)[i + u*width] = row_pointer[i];

			}
		break;

		case 3: // RGB picture
			
			*colour = 1;

			if (!(*greenimage = (unsigned int *)malloc(sizeof(unsigned int)*width*height)))
				return 1;
			if (!(*blueimage = (unsigned int *)malloc(sizeof(unsigned int)*width*height)))
				return 1;

			for (u=0; u<height; u++)
			{
				jpeg_read_scanlines(&cinfo, &row_pointer, 1);
				for (i=0; i<width; i++)
				{
					(*redimage)[i + u*width] = row_pointer[3*i];
					(*greenimage)[i + u*width] = row_pointer[3*i+1];
					(*blueimage)[i + u*width] = row_pointer[3*i+2];
				}
			}
		break;

		default:
			return 1;
	}

	jpeg_finish_decompress(&cinfo);

	// Release the JPEG decompression object.
	jpeg_destroy_decompress(&cinfo);
	
	*m = width;
	*n = height;

	if (row_pointer) free(row_pointer);
	close(infile);
	
	return(0);
}


/******************************************************************************/
/* Function used to read a BMP extension file                                 */
/******************************************************************************/
/* Creation : 26/08/2005 (M. TALLUR)                                          */
/******************************************************************************/

int read_bmp(char *filename, unsigned int **redimage, unsigned int **greenimage, unsigned int **blueimage, int *m, int *n, int *colour)
{
	FILE *f;
	bmpinfo B;
	char c1, c2, v;
	int offset, tmp, i, j, width, height, t=0;
	struct rgbColour *coloursTable = NULL;
	

	if ((f = fopen(filename, "r")))
	{
		// ENTETE FICHIER
 
		fread(&c1, 1, 1, f);
		fread(&c2, 1, 1, f);
		fread(&t, 4, 1, f);
		fread(&tmp, 4, 1, f);
		fread(&offset, 4, 1, f);
 
		// ENTETE BITMAP
 
		fread(&B.biSize, 4, 1, f);
		fread(&B.biWidth, 4, 1, f);
		fread(&B.biHeight, 4, 1, f);
		fread(&B.biPlanes, 2, 1, f);
		fread(&B.biBitCount, 2, 1, f);
		fread(&B.biCompression, 4, 1, f);
		fread(&B.biSizeImage, 4, 1, f);
		fread(&B.biXpelsPerMeter, 4, 1, f);
		fread(&B.biYpelsPerMeter, 4, 1, f);
		fread(&B.biClrUsed, 4, 1, f);
		fread(&B.biClrImportant, 4, 1, f);

		B.biBitCount = B.biBitCount & 0x0000FFFF;

		//sciprint("fichier %s taille %dx%d bitcount %d compression %d planes %d\n", filename, B.biWidth, B.biHeight, B.biBitCount, B.biCompression, B.biPlanes);

		if (B.biCompression)
		{
			sciprint("ImRead : unsupported BMP compression (RLE)\n");
			return(1);
		}


		width = B.biWidth;
		height = B.biHeight;

		//fseek(f, 10, SEEK_SET);
		//fread(&offset, 4, 1, f);
		fseek(f, 54, SEEK_SET);

		*colour = 1;
		
		if (!(*redimage = (unsigned int *)malloc(sizeof(unsigned int)*width*height)))
			return (1);
			
		if (!(*greenimage = (unsigned int *)malloc(sizeof(unsigned int)*width*height)))
			return (1);

		if (!(*blueimage = (unsigned int *)malloc(sizeof(unsigned int)*width*height)))
			return (1);
		

		// true color
		if (B.biBitCount == 24)
		{
			for(i=height-1; i>=0; i--)  
				for(j=0; j < B.biWidth; j++) 
				{ 
					fread(&v, 1, 1, f);
					(*blueimage)[j + i*width] = (unsigned char)v;
					fread(&v, 1, 1, f);
					(*greenimage)[j + i*width] = (unsigned char)v;
					fread(&v, 1, 1, f);
					(*redimage)[j + i*width] = (unsigned char)v;
				}
		}
		else
		// paletted 8-bit
		/*	if (B.biBitCount == 8)
			{
				int colourEntry, counter;
				*colour = 1;

				// let's read the colours table
				if (!(coloursTable = (struct rgbColour *)malloc(sizeof(struct rgbColour)*256)))
					return (1);
					
				for (i=0; i<256; i++)
				{
					fread(&colourEntry, 4, 1, f);
					coloursTable[i].blue = (colourEntry & 0x000000FF);
					coloursTable[i].green = (colourEntry & 0x0000FF00) >> 8;
					coloursTable[i].red = (colourEntry & 0x00FF0000) >> 16;
				}

				for(i=height-1; i>=0; i--)  
				{
					for(j=0; j < width; j++) 
					{ 
						sciprint("%d ", fread(&v, 1, 1, f));
						(*redimage)[j + i*width] = coloursTable[v].red;	
						(*greenimage)[j + i*width] = coloursTable[v].green;
						(*blueimage)[j + i*width] = coloursTable[v].blue;
					}

					for (counter = 0; counter < width%4 ; counter++)
						fread(&v, 1, 1, f);
				}
			}
			else*/
			{
				fclose(f);
				sciprint("ImRead : Unsupported BMP file depth (only 24 bits ATM)\n");
	    			return(1);
			}
      	}
	else
	{
		sciprint("ImRead : Cannot open BMP file\n");
		return (1);
	}

	fclose(f);

	*m = width;
	*n = height;
		
	return(0);
}
