
/******************************************************************************/
/* Fonctions used to carry out the vote with several classes images           */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "stack-c.h"
#include "int_imvote.h"


/******************************************************************************/
/* Interface Scilab<->imvote                                                  */
/******************************************************************************/
/* Creation : 27/05/2004 (V. POMMEREUL)                                       */
/* Last modification : 28/06/2004 (V. POMMEREUL)                              */
/******************************************************************************/
int int_imvote(char *fname) {

	// Input and output arguments
	static int l_dimension, m_dimension, n_dimension,
			   l_classnb, m_classnb, n_classnb,
			   l_nbim, m_nbim, n_nbim,
			   m_image, n_image,
			   m_final_classes_image, n_final_classes_image;
	static int l_images[NBIMAGES_MAX];
	double *l_final_classes_image;
	HyperMat h_images[NBIMAGES_MAX], h_final_classes_image;

	// Number of input and output arguments
	int minlhs = 1, maxlhs = 1;
	int minrhs = 5, maxrhs = 8;

	int ierr=0;

	int ind_var=1;
	int ind_res=1;

	int dim, classNb, nbIm;
	double *s_images[NBIMAGES_MAX];

	int i;


	// Verify number of arguments
	CheckRhs(minrhs, maxrhs); // Input arguments
	CheckLhs(minlhs, maxlhs); // Output arguments

	// Get right hand side arguments
	GetRhsVar(ind_var++, "i", &m_dimension, &n_dimension, &l_dimension);
	dim = *istk(l_dimension);

	GetRhsVar(ind_var++, "i", &m_classnb, &n_classnb, &l_classnb);
	classNb = *istk(l_classnb);

	GetRhsVar(ind_var++, "i", &m_nbim, &n_nbim, &l_nbim);
	nbIm = *istk(l_nbim);

	for (i=0 ; i<nbIm ; i++) {
		if (dim == 2) {
			GetRhsVar(ind_var++, "d", &m_image, &n_image, &l_images[i]);
		} else {
			GetRhsHMat(ind_var++, &h_images[i]);
		}
	}

	// Compute kmeans method classification phase
	if (dim == 2) {

		for (i=0 ; i<nbIm ; i++) {
			s_images[i] = stk(l_images[i]);
		}

		ierr = vote_2D(nbIm, classNb, s_images, m_image, n_image, &l_final_classes_image);

	} else {
		ierr = vote_3D_4D(nbIm, classNb, h_images, &h_final_classes_image);
	}


	// In case of error
	if(ierr!=0) {
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	// Return classes image
	if (dim == 2) {
		CreateVarFromPtr(ind_var, "d", &m_image, &n_image, &l_final_classes_image);
		if (l_final_classes_image) {
			free(l_final_classes_image);
		}
		LhsVar(ind_res)=ind_var;

	} else {

		CreateHMat(ind_var,&h_final_classes_image);
		LhsVar(ind_res) = ind_var;
		// Memory desallocation
		if (h_final_classes_image.dims) {
			free(h_final_classes_image.dims);
		}
		if (h_final_classes_image.R) {
			free(h_final_classes_image.R);
		}
	}

	return(0);

}


/******************************************************************************/
/* Vote function for a 2D image                                               */
/******************************************************************************/

int vote_2D(int nbIm, int classNb, double *images[NBIMAGES_MAX], int height, int width, double **final_classes_image) {

	int k;
	int err = 0;
	const int nbDim = 3;
	const int nbComp = 1;
	int dimvar = 0;
	HyperMat hfinalclim, himages[NBIMAGES_MAX];

	// ........................................................................
	// Create an hypermatrix array for images
	for (k=0 ; k<nbIm ; k++) {
		himages[k].dimsize = nbDim;
		dimvar = 0;
		if (himages[k].dims = (int *) calloc(himages[k].dimsize, sizeof(int))) {
			himages[k].dims[dimvar++] = height;
			himages[k].dims[dimvar++] = width;
			himages[k].dims[dimvar++] = nbComp;
		} else {
			return 1;
		}
		himages[k].size = height * width * nbComp;
		himages[k].R = images[k];
		himages[k].it = 0;
	}


	// ........................................................................
	// Memory allocation for the classes image
	if (*final_classes_image = (double*) calloc(height * width * nbComp, sizeof(double))) {
		// Create an hypermatrix for the classes image
		hfinalclim.dimsize = nbDim;
		dimvar = 0;
		if (hfinalclim.dims = (int *) calloc(hfinalclim.dimsize, sizeof(int))) {
			hfinalclim.dims[dimvar++] = height;
			hfinalclim.dims[dimvar++] = width;
			hfinalclim.dims[dimvar++] = nbComp;
		} else {
			return 1;
		}
		hfinalclim.size = height * width * nbComp;
		hfinalclim.R = (*final_classes_image);
		hfinalclim.it = 0;
	} else {
		return(1);
	}

	// ........................................................................
	// Filling classes image

	voter(nbIm, classNb, himages, hfinalclim);


	// ........................................................................
	// Memory desallocation
	for (k=0 ; k<nbIm ; k++) {
		if (himages[k].dims) {
			free(himages[k].dims);
		}
	}
	if (hfinalclim.dims) {
		free(hfinalclim.dims);
	}

	return(err);
}


/******************************************************************************/
/* Vote function for a 3D or 4D image                                         */
/******************************************************************************/

int vote_3D_4D(int nbIm, int classNb, HyperMat images[NBIMAGES_MAX], HyperMat *final_classes_image) {

	int i;
	int err = 0;
	const int nbDim = 3;
	const int nbComp = images[0].dims[3];
	int dimvar = 0;


	// ........................................................................
	// Filling classes image
	final_classes_image->size = images[0].size;
	final_classes_image->dimsize = images[0].dimsize;
	if (final_classes_image->dims = (int *) calloc(final_classes_image->dimsize, sizeof(int))) {
		for (i=0 ; i<final_classes_image->dimsize ; i++) {
			final_classes_image->dims[i] = images[0].dims[i];
		}
	} else {
		return 1;
	}
	if (!(final_classes_image->R = (double *) calloc(final_classes_image->size, sizeof(double)))) {
		return 1;
	}
	final_classes_image->it = 0;


	// ........................................................................

	voter(nbIm, classNb, images, *final_classes_image);

	return(err);
}


/******************************************************************************/
/* Vote function                                                              */
/******************************************************************************/

void voter(int nbIm, int classNb, HyperMat images[NBIMAGES_MAX], HyperMat final_classes_image) {

	int i_pixel, i_image, i_class;
	int *votes;
	int vote;
	int max, winner;

	if (!(votes = (int *)calloc(classNb+1, sizeof(int)))) {
		return;
	}


	for (i_pixel=0 ; i_pixel<final_classes_image.size ; i_pixel++) {

		memset(votes, 0, (classNb+1) * sizeof (int));

		for (i_image=0 ; i_image<nbIm ; i_image++) {
			vote = images[i_image].R[i_pixel];
			if (vote <= classNb) {
				votes[vote]++;
			}
		}

		// Search the max
		max = votes[0];
		winner = 0;
		for (i_class=1 ; i_class <= classNb ; i_class++) {
			if (votes[i_class] > max) {
				max = votes[i_class];
				winner = i_class;
			}
		}

		// Fill the final classes image
		final_classes_image.R[i_pixel] = winner;
	}

	if(votes) {
		free(votes);
	}

}






