/******************************************************************************/
/* Fonctions used to write a pgm image from Scilab                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>

#include "int_write_pgm.h"

#ifdef unix
#include <sys/stat.h>
#define O_BINARY       0x8000
#else
#include <sys\stat.h>
#endif
#include "stack-c.h"

/******************************************************************************/
/* Interface Scilab<->pgm_write                                               */
/******************************************************************************/
/* Creation : 22/03/2002 (V. COUVERT)                                         */
/* Last modification : 19/04/2002 (V. COUVERT)                                */
/******************************************************************************/
int int_write_pgm(char *fname)
{
	unsigned int l_filename, m_filename, n_filename,
		l_image, m_image, n_image,
		l,m=1,n=1;

	int minlhs=1, maxlhs=1, minrhs=2, maxrhs=2;

	int ierr=0;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */

	GetRhsVar(1, "c", &m_filename, &n_filename, &l_filename);
	GetRhsVar(2, "i", &m_image, &n_image, &l_image);

	pgm_write(cstk(l_filename), istk(l_image), m_image, n_image, &ierr);

	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}

	CreateVar(3,"i",&m,&n,&l);
	LhsVar(1)=3;
	return(0);
}


/******************************************************************************/
/* Function used to write a PGM extension file                                */
/* 8-bit grey level image                                                     */
/******************************************************************************/
/* Creation : 09/07/2001 (V. COUVERT)                                         */
/* Last modification : 12/02/2002 (V. COUVERT)                                */
/******************************************************************************/
int pgm_write(char *filename, unsigned int *image, int m, int n, int *ierr)
{
	int id; /* File identificator  */
	int image_width=m, image_height=n, nb_colors, x, y, i=0;
	char *magic="P5\n", c, buf[80], *commentary="#Scilab process result image\n";
	unsigned char *data;

	/* Try to create file */
	id=open(filename, O_WRONLY | O_CREAT | O_BINARY, S_IREAD | S_IWRITE);
	if(id==-1) /* If creation fails */
	{
		sciprint("Can't create file %s.\n", filename);
		return(0);
	}

	/* File header writing */
	write(id, magic, 3); /* Magic number */

	/* Comments writing */
	write(id, commentary, 29);

	/* Width and height writing */
	sprintf(buf, "%d %d\n", image_width, image_height);
	write(id, buf, strlen(buf));

	/* Colors writing */
	write(id, "255\n", 4);

	/* Memory allocation */
	if (!(data = (unsigned char*)calloc(1, sizeof(unsigned
 char)*image_width*image_height)))
	{
		return(1);
	}

	for(y=0;y<image_height;y++)
	{
		for(x=0;x<image_width;x++)
		{
			*(data+x+y*image_width)=*(image+x+y*image_width);
		}

	}
	write(id, data, image_width*image_height);
	close (id);

	free(data);
	return(0);
}


