/******************************************************************************/
/* Fonctions used to write a greyscales png image from Scilab                 */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>

#include "int_write_png_gray.h"
#include "png.h"

#include "stack-c.h"

/******************************************************************************/
/* Interface Scilab<->png_grey_write                                          */
/******************************************************************************/
/* Creation : 25/08/2005 (M. TALLUR)                                          */
/******************************************************************************/
int int_write_png_gray(char *fname)
{
	unsigned int l_filename, m_filename, n_filename,
		l_image, l_width, l_height,l, m=1, n=1,
		m_image, n_image,
		l_depth, m_depth, n_depth;

	int minlhs=1, maxlhs=1, minrhs=3, maxrhs=3;

	int ierr=0;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */

	GetRhsVar(1, "c", &m_filename, &n_filename, &l_filename);
	GetRhsVar(2, "i", &m_image, &n_image, &l_image);
	GetRhsVar(3, "i", &m_depth, &n_depth, &l_depth);

	png_gray_write(cstk(l_filename), istk(l_image), m_image, n_image, *istk(l_depth), &ierr);

	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}
	CreateVar(4,"i",&m,&n,&l);
	LhsVar(1)=4;
	return(0);
}

/******************************************************************************/
/* Function used to read a grayscales PNG extension file                      */
/* 1*8 or 1*16 bits grayscales image                                          */
/******************************************************************************/
/* Creation : 25/08/2005 (M. TALLUR)                                          */
/******************************************************************************/
int png_gray_write(char *filename, unsigned int *image, int m, int n, int depth, int *ierr)
{
	FILE *fp;
	png_structp png_ptr; 
	png_infop info_ptr; 
	png_bytep *row_pointers;
	int i, u, width, height;
	
	width = m; height = n;

	if (!(fp = fopen(filename, "wb")))
	{
		sciprint("ImWrite (PNG) : Can't open '%s' for output\n", filename);
		return(1); 
	}

	if (!(png_ptr = png_create_write_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL)))
		return (1);

	if (!(info_ptr = png_create_info_struct(png_ptr)))
		return (1);
		
	if (!info_ptr)
	{
		png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
		return (1);
	}

	if (setjmp(png_jmpbuf(png_ptr)))
	{
		/* If we get here, we had a problem reading the file */
		fclose(fp);
		png_destroy_write_struct(&png_ptr, &info_ptr);
		return (1);
	}
	
    	png_init_io(png_ptr, fp);
	
	info_ptr->width = width;
	info_ptr->height = height;
	info_ptr->rowbytes = width*depth; 
	info_ptr->valid = 1;
	info_ptr->bit_depth = depth;
	info_ptr->color_type = PNG_COLOR_TYPE_GRAY;
	info_ptr->compression_type = PNG_COMPRESSION_TYPE_DEFAULT;
	info_ptr->filter_type = PNG_FILTER_TYPE_DEFAULT;
	info_ptr->interlace_type = PNG_INTERLACE_NONE;
	info_ptr->num_palette = 0;
	info_ptr->num_trans = 0;
	
	info_ptr->channels = 1;
	info_ptr->pixel_depth = depth;
	info_ptr->spare_byte = 0;
	info_ptr->signature[0] = 0x89;
	info_ptr->signature[1] = 0x50;
	info_ptr->signature[2] = 0x4e;
	info_ptr->signature[3] = 0x47;
	info_ptr->signature[4] = 0x0d;
	info_ptr->signature[5] = 0x0a;
	info_ptr->signature[6] = 0x1a;
	info_ptr->signature[7] = 0x0a;
    
	if (!(row_pointers = (png_bytep *)malloc(sizeof(png_bytep)*height))) return 1;
	
	for (i=0; i<height; i++)
		if (!(row_pointers[i] = (png_bytep)malloc(width*(depth/8)))) return 1;
		
	if (depth == 8)
	{
		for (i=0; i<height; i++)
			for (u=0; u<width; u++)
			{ 
				row_pointers[i][u] = image[u + i*width];
			}
	}
	else
	{ 
		for (i=0; i<n; i++)
			for (u=0; u<m; u++)
			{
				row_pointers[i][2*u] = (image[u + i*width] & 0x0000FF00) >> 8;
				row_pointers[i][2*u+1] = (image[u + i*width] & 0x000000FF);
			}
	}	
	
	png_set_rows(png_ptr, info_ptr, row_pointers);
    
    	png_write_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL);
	
	png_destroy_write_struct(&png_ptr, &info_ptr);
	fclose(fp);
	
	return(0);
}

