/******************************************************************************/
/* Fonctions used to write a ppm image from Scilab                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>

#include "int_write_ppm.h"

#ifdef unix
#include <sys/stat.h>
#define O_BINARY       0x8000
#else
#include <sys\stat.h>
#endif
#include "stack-c.h"

/******************************************************************************/
/* Interface Scilab<->ppm_write                                               */
/******************************************************************************/
/* Creation : 25/03/2002 (V. COUVERT)                                         */
/* Last modification : 19/04/2002 (V. COUVERT)                                */
/******************************************************************************/
int int_write_ppm(char *fname)
{
	unsigned int l_filename, m_filename, n_filename,
		l_redimage, l_width, l_height,l, l_greenimage, l_blueimage, m=1, n=1,
		m_redimage, n_redimage,
		m_greenimage, n_greenimage,
		m_blueimage, n_blueimage;

	int minlhs=1, maxlhs=1, minrhs=4, maxrhs=4;

	int ierr=0;

	CheckRhs(minrhs, maxrhs); /* Input arguments */
	CheckLhs(minlhs, maxlhs); /* Output arguments */

	GetRhsVar(1, "c", &m_filename, &n_filename, &l_filename);
	GetRhsVar(2, "i", &m_redimage, &n_redimage, &l_redimage);
	GetRhsVar(3, "i", &m_greenimage, &n_greenimage, &l_greenimage);
	GetRhsVar(4, "i", &m_blueimage, &n_blueimage, &l_blueimage);

	ppm_write(cstk(l_filename), istk(l_redimage), istk(l_greenimage), istk(l_blueimage), m_redimage, n_redimage, &ierr);

	/* In case of error */
	if(ierr>0)
	{
		Scierror(999,"%s : Internal error\r\n",fname);
	}
	CreateVar(5,"i",&m,&n,&l);
	LhsVar(1)=5;
	return(0);
}

/******************************************************************************/
/* Function used to read a PPM extension file                                 */
/* 24-bit color image                                                         */
/******************************************************************************/
/* Creation : 25/03/2002 (V. COUVERT)                                         */
/* Last modification : 25/032002 (V. COUVERT)                                */
/******************************************************************************/
int ppm_write(char *filename, unsigned int *redimage, unsigned int *greenimage, unsigned int *blueimage, int m, int n, int *ierr)
{
	int id; /* File identificator  */
	int image_width=m, image_height=n, nb_colors, x, y, i=0;
	char *magic="P6\n", c, buf[80], *commentary="#Scilab process result image\n";
	unsigned char *data;

	/* Try to create file */
	id=open(filename, O_WRONLY | O_CREAT | O_BINARY, S_IREAD | S_IWRITE);
	if(id==-1) /* If creation fails */
	{
		sciprint("Can't create file %s.\n", filename);
		*ierr=1;
		return(1);
	}

	/* File header writing */
	write(id, magic, 3); /* Magic number */

	/* Comments writing */
	write(id, commentary, 29);

	/* Width and height writing */
	sprintf(buf, "%d %d\n", image_width, image_height);
	write(id, buf, strlen(buf));

	/* Colors writing */
	write(id, "255\n", 4);


    	/* Memory allocation */
    	if (!(data = (unsigned char*)calloc(1, 3*sizeof(unsigned char)*image_width*image_height)))
    	{
		*ierr=1;
		return(1);
    	}

	for(y=0 ;y<image_height ; y++)
    	{
      		for(x=0 ; x<image_width ; x++)
      		{
      			*(data+3*(x+y*image_width))=redimage[x+y*image_width];
      			*(data+3*(x+y*image_width)+1)=greenimage[x+y*image_width];
      			*(data+3*(x+y*image_width)+2)=blueimage[x+y*image_width];
      		}
      	}

    	/* Data writing */
	write(id, data, 3*image_width*image_height);

	close (id);

	free(data);

	return(0);
}

