

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "fuzzykmeanslib.h"


// FuzzyKmeans object
FuzzyKmeans *fuzzykmeans;


void fuzzyKmeansInit(int nNbComp, int nClassNumber, int nInputDimension) {
	// Creation of Kmeans object
	fuzzykmeans = new FuzzyKmeans(nNbComp, nClassNumber, nInputDimension);
}

void fuzzyKmeansDeinit() {
	// Destruction of FuzzyKmeans object
	if (fuzzykmeans)
		delete fuzzykmeans;
}

//...................................................................................................

int fuzzyKmeansTrain(HyperMat learningData, HyperMat proto, double threshold, int nbIter, double fuzzyness) {

	// Fuzzy k-means method training phase
	return fuzzykmeans->fuzzyTrain(learningData, proto, threshold, nbIter, fuzzyness);
}


int fuzzyKmeansClassify(HyperMat proto, HyperMat mask, double *weights, double fuzzyness, double threshold1,
						double threshold2, HyperMat images[NBIMAGES_MAX], HyperMat classes_image ) {

	// Fuzzy k-means method classification phase
	return fuzzykmeans->fuzzyClassify(proto, mask, weights, fuzzyness, threshold1, threshold2, images, classes_image);
}

//...................................................................................................


FuzzyKmeans::FuzzyKmeans(int nNbComp, int nClassNumber, int nInputDimension) :
				NeuralNet(nNbComp, nClassNumber, nInputDimension) {}


FuzzyKmeans::~FuzzyKmeans() {}


//################################################################################################################



int FuzzyKmeans::fuzzyTrain(HyperMat learningData, HyperMat proto, double stopThreshold, int nbIter, double fuzzyness) {

	int nbPoints= learningData.dims[0];
	int nbComp = proto.dims[2];
	double invFuzzyness_1;
	double **classBelonging; // classBelonging[classNb][pointNb] : degree of membership
	double *distances;
	double rError;
	double rAux;
	int i_fun, i_point, i_comp, i_class;
	int iEpoch;
	bool clear = false;
	bool bChanged = true;


	if (!nbPoints) {
		return 1;
	}

	if (fuzzyness != 2.0) {
		invFuzzyness_1 = 1.0 / (fuzzyness - 1.0);
	}

	if (flags & GENERIC_NN_PROTO) {

		// Distance vector creation
		distances = (double *) calloc (classCount, sizeof (double));
		if (distances) {

			// Degree of membership matrix creation
			classBelonging = (double **) calloc (classCount, sizeof(double *));
			if (classBelonging) {
				classBelonging[0] = (double *) calloc (classCount * nbPoints, sizeof(double));
				if (classBelonging[0]) {
					for (i_class = 1; i_class < classCount; i_class++) {
						classBelonging[i_class] = classBelonging[0] + i_class * nbPoints;
					}

					//Prototype initialisation
					initProto(learningData, proto);


					iEpoch = 0;
					do {
						// Degree of membership matrix computation
						for (i_point = 0; i_point < nbPoints; i_point++) {

							// Distances between the point and the prototype
							for (i_class = 0; i_class < classCount; i_class++) {
								distances[i_class] = 0;

								for (i_fun = 0; i_fun < inputSpaceDim; i_fun++) {
									for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
										rAux = LearningData(i_point, i_fun, i_comp) - Proto(i_fun, i_class, i_comp);
										distances[i_class] += rAux * rAux ;
									}
								}
							}

							// Fuzzy degree of membership computation in each class
							rAux = 0.0;
							for (i_class = 0; i_class < classCount; i_class++) {
								if (distances[i_class] != 0) {
									if (fuzzyness == 2.0) {
										rAux += 1.0 / distances[i_class];
									} else {
										rAux += 1.0 / pow (distances[i_class], invFuzzyness_1);
									}
								} else {
									clear = true;
								}
							}

							for (i_class = 0; i_class < classCount; i_class++) {
								if (!clear) {
									if (fuzzyness == 2.0) {
										classBelonging[i_class][i_point] = 1.0 / (rAux * distances[i_class]);
									} else {
										classBelonging[i_class][i_point] = 1.0 /
											(pow (distances[i_class], invFuzzyness_1) * rAux);
									}
								} else {
									if (distances[i_class] == 0) {
										classBelonging[i_class][i_point] = 1.0;
									} else {
										classBelonging[i_class][i_point] = 0.0;
									}
								}
							}
							clear = false;
						}

						// Update prototype
						for (i_class = 0; i_class < classCount; i_class++) {
							for (i_fun = 0; i_fun < inputSpaceDim; i_fun++) {
								for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
									PrivateProto(i_fun, i_class, i_comp) = 0.0;
								}
							}
						}
						for (i_class = 0; i_class < classCount; i_class++) {
							rAux = 0;
							for (i_point = 0; i_point < nbPoints; i_point++) {

								if (fuzzyness == 2.0) {
									rAux += classBelonging[i_class][i_point] * classBelonging[i_class][i_point];
								} else {
									rAux += pow (classBelonging[i_class][i_point], fuzzyness);
								}

								for (i_fun = 0; i_fun < inputSpaceDim; i_fun++) {
									for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
										if (fuzzyness == 2.0) {
											PrivateProto(i_fun, i_class, i_comp)
												+= 	classBelonging[i_class][i_point] *
													classBelonging[i_class][i_point] *
													LearningData(i_point, i_fun, i_comp);
										} else {
											PrivateProto(i_fun, i_class, i_comp)
											+= 	pow (classBelonging[i_class][i_point], fuzzyness) *
												LearningData(i_point, i_fun, i_comp);
										}
									}
								}
							}

							for (i_fun = 0; i_fun < inputSpaceDim; i_fun++) {
								for (i_comp = 0 ; i_comp < nbComp ; i_comp++) {
									if (rAux!=0.0) {
										PrivateProto(i_fun, i_class, i_comp) = PrivateProto(i_fun, i_class, i_comp) /rAux;
									} // else : no change
								}
							}
						}

						// difference between prototypes of 2 consecutive computations
						rError = 0.0;
						for (i_class = 0; i_class < classCount; i_class++) {
							for (i_fun = 0; i_fun < inputSpaceDim; i_fun++) {
								for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {

									rAux = Proto(i_fun, i_class, i_comp) - PrivateProto(i_fun, i_class, i_comp);
									Proto(i_fun, i_class, i_comp) = PrivateProto(i_fun, i_class, i_comp);
									rError += rAux * rAux;
								}
							}
						}
						rError = sqrt (rError);
						bChanged = (rError > stopThreshold);
						iEpoch++;

					} while (bChanged || (iEpoch < nbIter));

					free(classBelonging[0]);
				}
				free(classBelonging);
			}
			free(distances);
		}
	} //end if (flags & GENERIC_NN_PROTO)

	// Sort prototype classes in increasing order
	sortProto(proto);

	return 0;
}


//################################################################################################################


int FuzzyKmeans::fuzzyClassify(HyperMat proto, HyperMat mask, double *weights, double fuzzyness, double threshold1,
						double threshold2, HyperMat images[NBIMAGES_MAX], HyperMat classes_image) {

	double **value;
	int i_point, i_fun, i_comp;

	// Memory allocation
	if (value = (double **) calloc(inputSpaceDim, sizeof(double*))) {
		if (value[0] = (double *) calloc(inputSpaceDim * nbComp, sizeof(double))) {
			for (i_fun=1 ; i_fun<inputSpaceDim ; i_fun++) {
				value[i_fun] = value[0] + i_fun * nbComp;
			}
		} else {
			return 1;
		}
	} else {
		return 1;
	}


	// Classify each pixel of images and form a classes image
	for (i_point=0 ; i_point<mask.size ; i_point++) {
		if (mask.R[i_point]) {
			for (i_fun=0 ; i_fun<inputSpaceDim ; i_fun++) {
				for (i_comp=0 ; i_comp < nbComp ; i_comp++) {
					value[i_fun][i_comp] = weights[i_fun] * images[i_fun].R[i_point + i_comp * mask.size];
				}
			}
			classes_image.R[i_point] = (double) classifyValue(proto, value, fuzzyness, threshold1, threshold2);
		} else {
			classes_image.R[i_point] = 0.0;
		}
	}

	// Free memory
	if (value) {
		if (value[0]) {
			free(value[0]);
		}
		free(value);
	}

	return 0;
}

//...................................................................................................


int FuzzyKmeans::classifyValue(HyperMat proto, double **value, double fuzzyness, double threshold1, double threshold2) {

	int i_fun, i_class, i_comp ;
	double rAux;
	double rMax[2];
	double *distances, *classBelonging;
	double invFuzzyness_1 = 1.0 / (fuzzyness - 1.0);
	bool clear = false;
	int iIndex[2];
	int iResult = classCount;

	if (flags & GENERIC_NN_PROTO) {
		distances = (double *) calloc (classCount, sizeof (double));
		if (distances) {

			classBelonging = (double *) calloc (classCount, sizeof (double));
			if (classBelonging) {

				// Distances computation
				for (i_class = 0; i_class < classCount; i_class++) {
					distances[i_class] = 0;
					for (i_fun = 0 ; i_fun < inputSpaceDim; i_fun++) {
						for (i_comp = 0 ; i_comp < nbComp ; i_comp ++) {
							rAux = value[i_fun][i_comp] - Proto(i_fun, i_class, i_comp);
							distances[i_class] += rAux * rAux;
						}
					}
				}


				// Fuzzy degree of membership computation in each class
				rAux = 0.0;
				for (i_class = 0; i_class < classCount; i_class++) {
					if (distances[i_class] != 0) {
						rAux += 1.0 / pow (distances[i_class], invFuzzyness_1);
					} else {
						clear = true;
					}
				}
				for (i_class = 0; i_class < classCount; i_class++) {
					if (!clear) {
						classBelonging[i_class] = 1.0 /
							(pow (distances[i_class], invFuzzyness_1) * rAux);
					} else {
						if (distances[i_class] == 0) {
							classBelonging[i_class] = 1.0;
						} else {
							classBelonging[i_class] = 0.0;
						}
					}
				}

				rMax[0] = classBelonging[0];
				rMax[1] = rMax[0];
				iIndex[0] = 0;
				iIndex[1] = 0;

				for (i_class = 0; i_class < classCount; i_class++) {
					if (rMax[0] < classBelonging[i_class]) {
						rMax[0] = classBelonging[i_class];
						iIndex[0] = i_class;
					}
					if ((rMax[1] < classBelonging[i_class]) && (rMax[0] > classBelonging[i_class])) {
						rMax[1] = classBelonging[i_class];
						iIndex[1] = i_class;
					}
				}

				if (rMax[0] > threshold1) {
					iResult = iIndex[0];
				} else if (rMax[0] - rMax[1] > threshold2) {
					iResult = iIndex[0];
				}
				//else iResult = classCount

				free (classBelonging);
			}
			free(distances);
		}
	}

	return iResult;

}



